@@ -46,30 +46,17 @@
 /* Build version */
 #define PROG    "genhash"
 
-#define VERSION_CODE 0x010000
-#define DATE_CODE    0x15070d
-
-#define GETMETER_VERSION(version)	\
-	(version >> 16) & 0xFF,		\
-	(version >> 8) & 0xFF,		\
-	version & 0xFF
-
-#ifdef VERSION_STRING
-  #undef VERSION_STRING
-#endif
-#define VERSION_STRING PROG" v%d.%d.%d (%.2d/%.2d, 20%.2d)\n",	\
-		GETMETER_VERSION(VERSION_CODE),			\
-		GETMETER_VERSION(DATE_CODE)
-
 /* HTTP/HTTPS request structure */
 typedef struct {
 	struct		addrinfo *dst;
 	char		ipaddress[INET6_ADDRSTRLEN];
 	uint16_t	addr_port;
-	char		*url;
-	char		*vhost;
+	const char	*url;
+	const char	*vhost;
 	int		verbose;
 	int		ssl;
+	http_protocol_t http_protocol;
+	unsigned	timeout;
 #ifdef _HAVE_SSL_SET_TLSEXT_HOST_NAME_
 	int		sni;
 #endif
@@ -84,7 +71,6 @@ typedef struct {
 } REQ;
 
 /* Global variables */
-extern thread_master_t *master;
 extern REQ *req;		/* Cmd line arguments */
 extern int exit_code;
 
