@@ -31,9 +31,8 @@
 
 /* Prototypes */
 extern void init_ssl(void);
-extern bool ssl_connect(thread_t *);
-extern int ssl_printerr(int);
-extern int ssl_send_request(SSL *, char *, int);
-extern int ssl_read_thread(thread_t *);
+extern bool ssl_connect(thread_ref_t);
+extern int ssl_send_request(SSL *, const char *, int);
+extern int ssl_read_thread(thread_ref_t);
 
 #endif
