@@ -38,12 +38,13 @@ tcp_connect(int fd, REQ * req_obj)
 	struct sockaddr_in adr_serv;
 	struct sockaddr_in6 adr_serv6;
 	int ret;
-	int val;
 
-	/* free the tcp port after closing the socket descriptor */
+	/* free the tcp port after closing the socket descriptor, but allow
+	 * time for a proper shutdown. */
 	li.l_onoff = 1;
-	li.l_linger = 0;
-	setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *) &li, sizeof (struct linger));
+	li.l_linger = 5;
+	if (setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *) &li, sizeof (struct linger)))
+		fprintf(stderr, "Error setting SO_LINGER on socket %d\n", fd);
 
 #ifdef _WITH_SO_MARK_
 	if (req->mark) {
@@ -54,9 +55,6 @@ tcp_connect(int fd, REQ * req_obj)
 		}
 	}
 #endif
-	/* Make socket non-block. */
-	val = fcntl(fd, F_GETFL, 0);
-	fcntl(fd, F_SETFL, val | O_NONBLOCK);
 
 	if(req_obj->dst && req_obj->dst->ai_family == AF_INET6) {
 		long_inet = sizeof (struct sockaddr_in6);
@@ -79,24 +77,19 @@ tcp_connect(int fd, REQ * req_obj)
 	}
 
 	/* Immediate success */
-	if (ret == 0) {
-		fcntl(fd, F_SETFL, val);
+	if (ret == 0)
 		return connect_success;
-	}
 
-	/* If connect is in progress then return 1 else it's real error. */
-	if (ret < 0) {
-		if (errno != EINPROGRESS)
-			return connect_error;
-	}
+	/* If connect is in progress then return connect_in_progress else it's real error. */
+	if (errno != EINPROGRESS)
+		return connect_error;
 
 	/* restore previous fd args */
-	fcntl(fd, F_SETFL, val);
 	return connect_in_progress;
 }
 
 static enum connect_result
-tcp_socket_state(thread_t * thread, int (*func) (thread_t *))
+tcp_socket_state(thread_ref_t thread, thread_func_t func)
 {
 	int status;
 	socklen_t slen;
@@ -114,12 +107,12 @@ tcp_socket_state(thread_t * thread, int (*func) (thread_t *))
 	/* Check file descriptor */
 	slen = sizeof (status);
 	if (getsockopt
-	    (thread->u.fd, SOL_SOCKET, SO_ERROR, (void *) &status, &slen) < 0)
+	    (thread->u.f.fd, SOL_SOCKET, SO_ERROR, (void *) &status, &slen) < 0)
 		ret = errno;
 
 	/* Connection failed !!! */
 	if (ret) {
-		DBG("TCP connection failed to [%s]:%d.\n",
+		DBG("TCP getsockopt() failed to [%s]:%d.\n",
 		    req->ipaddress, ntohs(req->addr_port));
 		thread_close_fd(thread);
 		return connect_error;
@@ -130,22 +123,27 @@ tcp_socket_state(thread_t * thread, int (*func) (thread_t *))
 	 * and other error code until connection is established.
 	 * Recompute the write timeout (or pending connection).
 	 */
-	if (status != 0) {
+	if (status == EINPROGRESS) {
 		DBG("TCP connection to [%s]:%d still IN_PROGRESS.\n",
 		    req->ipaddress, ntohs(req->addr_port));
 
 		timer_min = timer_sub_now(thread->sands);
 		thread_add_write(thread->master, func, THREAD_ARG(thread)
-				 , thread->u.fd, timer_long(timer_min));
+				 , thread->u.f.fd, timer_long(timer_min), true);
 		return connect_in_progress;
+	} else if (status) {
+		DBG("TCP connection failed to [%s]:%d.\n",
+		    req->ipaddress, ntohs(req->addr_port));
+		thread_close_fd(thread);
+		return connect_error;
 	}
 
 	return connect_success;
 }
 
 static void
-tcp_connection_state(int fd, enum connect_result status, thread_t * thread,
-		     int (*func) (thread_t *)
+tcp_connection_state(int fd, enum connect_result status, thread_ref_t thread
+		     , thread_func_t func
 		     , unsigned long timeout)
 {
 	switch (status) {
@@ -156,13 +154,13 @@ tcp_connection_state(int fd, enum connect_result status, thread_t * thread,
 
 	case connect_success:
 		thread_add_write(thread->master, func, THREAD_ARG(thread),
-				 fd, timeout);
+				 fd, timeout, true);
 		break;
 
 		/* Checking non-blocking connect, we wait until socket is writable */
 	case connect_in_progress:
 		thread_add_write(thread->master, func, THREAD_ARG(thread),
-				 fd, timeout);
+				 fd, timeout, true);
 		break;
 
 	default:
@@ -171,7 +169,7 @@ tcp_connection_state(int fd, enum connect_result status, thread_t * thread,
 }
 
 static int
-tcp_check_thread(thread_t * thread)
+tcp_check_thread(thread_ref_t thread)
 {
 	SOCK *sock_obj = THREAD_ARG(thread);
 	int ret = 1;
@@ -197,6 +195,10 @@ tcp_check_thread(thread_t * thread)
 				ret = ssl_connect(thread);
 
 			if (ret) {
+				/* SSL connections manage their own threads for SSL_connect */
+				if (req->ssl)
+					return 1;
+
 				/* Remote WEB server is connected.
 				 * Unlock eventual locked socket.
 				 */
@@ -208,10 +210,8 @@ tcp_check_thread(thread_t * thread)
 				DBG("Connection trouble to: [%s]:%d.\n",
 				    req->ipaddress,
 				    ntohs(req->addr_port));
-				if (req->ssl)
-					ssl_printerr(SSL_get_error
-						     (sock_obj->ssl, ret));
 				sock_obj->status = connect_error;
+				thread_add_terminate_event(thread->master);
 				return -1;
 			}
 		}
@@ -222,12 +222,12 @@ tcp_check_thread(thread_t * thread)
 }
 
 int
-tcp_connect_thread(thread_t * thread)
+tcp_connect_thread(thread_ref_t thread)
 {
 	SOCK *sock_obj = THREAD_ARG(thread);
 
 	if ((sock_obj->fd = socket((req->dst && req->dst->ai_family == AF_INET6) ? AF_INET6 : AF_INET,
-				   SOCK_STREAM
+				   SOCK_STREAM | SOCK_NONBLOCK
 #ifdef SOCK_CLOEXEC
 					       | SOCK_CLOEXEC
 #endif
@@ -236,14 +236,15 @@ tcp_connect_thread(thread_t * thread)
 		return 0;
 	}
 
-#ifndef SOCK_CLOEXEC
-	fcntl(sock_obj->fd, F_SETFD, fcntl(sock_obj->fd, F_GETFD) | FD_CLOEXEC);
+#if !HAVE_DECL_SOCK_NONBLOCK
+	if (fcntl(sock_obj->fd, F_SETFL, fcntl(sock_obj->fd, F_GETFL) | O_NONBLOCK))
+		fprintf(stderr, "Unable to set socket non blocking\n");
 #endif
 
 	sock->status = tcp_connect(sock_obj->fd, req);
 
 	/* handle tcp connection status & register check worker thread */
 	tcp_connection_state(sock_obj->fd, sock_obj->status, thread, tcp_check_thread,
-			     HTTP_CNX_TIMEOUT);
+			     req->timeout);
 	return 0;
 }
