@@ -5,8 +5,6 @@
  *
  * Part:        Main entry point.
  *
- * Version:     $Id: main.c,v 1.1.16 2009/02/14 03:25:07 acassen Exp $
- *
  * Authors:     Alexandre Cassen, <acassen@linux-vs.org>
  *
  *              This program is distributed in the hope that it will be useful,
@@ -19,24 +17,22 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
 /* system includes */
-#include <signal.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/types.h>
 #include <stdio.h>
-#include <string.h>
 #include <sys/socket.h>
-#include <netdb.h>
+#include <arpa/inet.h>
 
 /* keepalived includes */
 #include "utils.h"
 #include "signals.h"
+#include "git-commit.h"
 
 /* genhash includes */
 #include "include/main.h"
@@ -44,6 +40,7 @@
 
 /* global var */
 REQ *req = NULL;
+int exit_code;
 
 /* Terminate handler */
 static void
@@ -57,7 +54,6 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 static void
 signal_init(void)
 {
-	signal_handler_init();
 	signal_set(SIGHUP, sigend, NULL);
 	signal_set(SIGINT, sigend, NULL);
 	signal_set(SIGTERM, sigend, NULL);
@@ -70,26 +66,37 @@ usage(const char *prog)
 {
 	enum feat_hashes i;
 
-	fprintf(stderr, VERSION_STRING);
-	fprintf(stderr,
-		"Usage:\n"
-		"  %s -s server-address -p port -u url\n"
-		"  %s -S -s server-address -p port -u url\n"
-		"  %s -h\n" "  %s -r\n\n", prog, prog, prog, prog);
+	fprintf(stderr, "%s", VERSION_STRING);
+#ifdef GIT_COMMIT
+	fprintf(stderr, ", git commit %s", GIT_COMMIT);
+#endif
+	fprintf(stderr, "\n\n%s\n\n", COPYRIGHT_STRING);
 	fprintf(stderr,
+		"Usage: %s COMMAND [OPTIONS]\n"
 		"Commands:\n"
+		"   -s server-address -p port -u url\n"
+		"   -S -s server-address -p port -u url\n"
+		"   -h\n"
+		"   -r\n\n", prog);
+	fprintf(stderr,
+		"Options:\n"
 		"Either long or short options are allowed.\n"
-		"  %s --use-ssl         -S       Use SSL connection to remote server.\n"
-		"  %s --server          -s       Use the specified remote server address.\n"
-		"  %s --port            -p       Use the specified remote server port.\n"
-		"  %s --url             -u       Use the specified remote server url.\n"
-		"  %s --use-virtualhost -V       Use the specified virtualhost in GET query.\n"
-		"  %s --hash            -H       Use the specified hash algorithm.\n"
-		"  %s --verbose         -v       Use verbose mode output.\n"
-		"  %s --help            -h       Display this short inlined help screen.\n"
-		"  %s --release         -r       Display the release number.\n"
-		"  %s --fwmark          -m       Use the specified FW mark.\n",
-		prog, prog, prog, prog, prog, prog, prog, prog, prog, prog);
+		"   --use-ssl         -S       Use SSL connection to remote server.\n"
+#ifdef _HAVE_SSL_SET_TLSEXT_HOST_NAME_
+		"   --use-sni         -I       Use SNI during SSL handshake (uses virtualhost setting; see -V).\n"
+#endif
+		"   --server          -s       Use the specified remote server address.\n"
+		"   --port            -p       Use the specified remote server port.\n"
+		"   --url             -u       Use the specified remote server url.\n"
+		"   --use-virtualhost -V       Use the specified virtualhost in GET query.\n"
+		"   --hash            -H       Use the specified hash algorithm.\n"
+		"   --verbose         -v       Use verbose mode output.\n"
+		"   --help            -h       Display this short inlined help screen.\n"
+		"   --release         -r       Display the release number.\n"
+		"   --fwmark          -m       Use the specified FW mark.\n"
+		"   --protocol        -P       Use the specified HTTP protocol - '1.0', 1.0c', '1.1'.\n"
+		"                                1.0c means 1.0 with 'Connection: close'\n"
+		"   --timeout         -t       Timeout in seconds\n");
 	fprintf(stderr, "\nSupported hash algorithms:\n");
 	for (i = hash_first; i < hash_guard; i++)
 		fprintf(stderr, "  %s%s\n",
@@ -105,6 +112,9 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 	struct addrinfo hint, *res = NULL;
 	int ret;
 	void *ptr;
+	char *endptr;
+	long port_num;
+	const char *start;
 
 	memset(&hint, '\0', sizeof hint);
 
@@ -112,24 +122,37 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 	hint.ai_flags = AI_NUMERICHOST;
 
 	struct option long_options[] = {
-		{"release",         no_argument,       0, 'r'},
-		{"help",            no_argument,       0, 'h'},
-		{"verbose",         no_argument,       0, 'v'},
-		{"use-ssl",         no_argument,       0, 'S'},
-		{"server",          required_argument, 0, 's'},
-		{"hash",            required_argument, 0, 'H'},
-		{"use-virtualhost", required_argument, 0, 'V'},
-		{"port",            required_argument, 0, 'p'},
-		{"url",             required_argument, 0, 'u'},
-		{"fwmark",          required_argument, 0, 'm'},
+		{"release",		no_argument,       0, 'r'},
+		{"help",		no_argument,       0, 'h'},
+		{"verbose",		no_argument,       0, 'v'},
+		{"use-ssl",		no_argument,       0, 'S'},
+#ifdef _HAVE_SSL_SET_TLSEXT_HOST_NAME_
+		{"use-sni",		no_argument,       0, 'I'},
+#endif
+		{"server",		required_argument, 0, 's'},
+		{"hash",		required_argument, 0, 'H'},
+		{"use-virtualhost",	required_argument, 0, 'V'},
+		{"port",		required_argument, 0, 'p'},
+		{"url",			required_argument, 0, 'u'},
+		{"fwmark",		required_argument, 0, 'm'},
+		{"protocol",		required_argument, 0, 'P'},
+		{"timeout",		required_argument, 0, 't'},
 		{0, 0, 0, 0}
 	};
 
 	/* Parse the command line arguments */
-	while ((c = getopt_long (argc, argv, "rhvSs:H:V:p:u:m:", long_options, NULL)) != EOF) {
+	while ((c = getopt_long (argc, argv, "rhvSs:H:V:p:u:m:P:t:"
+#ifdef _HAVE_SSL_SET_TLSEXT_HOST_NAME_
+							       "I"
+#endif
+				  , long_options, NULL)) != EOF) {
 		switch (c) {
 		case 'r':
-			fprintf(stderr, VERSION_STRING);
+			fprintf(stderr, "%s", VERSION_STRING);
+#ifdef GIT_COMMIT
+			fprintf(stderr, ", git commit %s", GIT_COMMIT);
+#endif
+			fprintf(stderr, "\n");
 			break;
 		case 'h':
 			usage(argv[0]);
@@ -140,6 +163,11 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 		case 'S':
 			req_obj->ssl = 1;
 			break;
+#ifdef _HAVE_SSL_SET_TLSEXT_HOST_NAME_
+		case 'I':
+			req_obj->sni = 1;
+			break;
+#endif
 		case 's':
 			if ((ret = getaddrinfo(optarg, NULL, &hint, &res)) != 0){
 				fprintf(stderr, "server should be an IP, not %s\n", optarg);
@@ -155,6 +183,7 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 					inet_ntop (res->ai_family, ptr, req_obj->ipaddress, INET6_ADDRSTRLEN);
 				} else {
 					fprintf(stderr, "server should be an IP, not %s\n", optarg);
+					freeaddrinfo(res);
 					return CMD_LINE_ERROR;
 				}
 			}
@@ -174,19 +203,55 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 			req_obj->vhost = optarg;
 			break;
 		case 'p':
-			req_obj->addr_port = htons(atoi(optarg));
+			port_num = strtol(optarg, &endptr, 10);
+			if (*endptr || port_num <= 0 || port_num > 65535) {
+				fprintf(stderr, "invalid port number '%s'\n", optarg);
+				return CMD_LINE_ERROR;
+			}
+			req_obj->addr_port = htons(port_num);
 			break;
 		case 'u':
 			req_obj->url = optarg;
 			break;
 		case 'm':
 #ifdef _WITH_SO_MARK_
-			req_obj->mark = (unsigned)strtoul(optarg, NULL, 10);
+			start = optarg + strspn(optarg, " \t");
+			req_obj->mark = (unsigned)strtoul(start, &endptr, 10);
+			if (*endptr || start[0] == '-' || start[0] == ' ') {
+				fprintf(stderr, "invalid fwmark '%s'\n", optarg);
+				return CMD_LINE_ERROR;
+			}
 #else
 			fprintf(stderr, "genhash built without fwmark support\n");
 			return CMD_LINE_ERROR;
 #endif
 			break;
+		case 'P':
+			if (!strcmp(optarg, "1.0"))
+				req_obj->http_protocol = HTTP_PROTOCOL_1_0;
+			else if (!strcmp(optarg, "1.0c") || !strcmp(optarg, "1.0C"))
+				req_obj->http_protocol = HTTP_PROTOCOL_1_0C;
+			else if (!strcmp(optarg, "1.1"))
+				req_obj->http_protocol = HTTP_PROTOCOL_1_1;
+			/* 1.0k and 1.1k are for test purposes and are not expected to be used */
+			else if (!strcmp(optarg, "1.0k"))
+				req_obj->http_protocol = HTTP_PROTOCOL_1_0K;
+			else if (!strcmp(optarg, "1.1k"))
+				req_obj->http_protocol = HTTP_PROTOCOL_1_1K;
+			else {
+				fprintf(stderr, "invalid HTTP protocol version '%s'\n", optarg);
+				return CMD_LINE_ERROR;
+			}
+			break;
+		case 't':
+			start = optarg + strspn(optarg, " \t");
+			req_obj->timeout = (unsigned)strtoul(start, &endptr, 10);
+			if (*endptr || start[0] == '-' || !start[0]) {
+				fprintf(stderr, "invalid timeout '%s'\n", optarg);
+				return CMD_LINE_ERROR;
+			}
+			req_obj->timeout *= TIMER_HZ;
+			break;
 		default:
 			usage(argv[0]);
 			return CMD_LINE_ERROR;
@@ -208,49 +273,50 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 int
 main(int argc, char **argv)
 {
-	thread_t thread;
-	char *url_default = malloc(2);
-	url_default[0] = '/';
-	url_default[1] = '\0';
+	const char *url_default = "/";
+
+#ifdef _MEM_CHECK_
+	mem_log_init("Genhash", "Genhash process");
+	enable_mem_log_termination();
+#endif
 
 	/* Allocate the room */
 	req = (REQ *) MALLOC(sizeof (REQ));
 
 	/* Preset (potentially) non-zero defaults */
 	req->hash = hash_default;
+	req->http_protocol = HTTP_PROTOCOL_1_0;
+	req->timeout = HTTP_CNX_TIMEOUT * TIMER_HZ;
 
 	/* Command line parser */
 	if (!parse_cmdline(argc, argv, req)) {
-		FREE(url_default);
 		FREE(req);
-		exit(0);
+		exit(1);
 	}
 
 	/* Check minimum configuration need */
 	if (!req->dst && !req->addr_port && !req->url) {
-		FREE(url_default);
 		freeaddrinfo(req->dst);
 		FREE(req);
-		exit(0);
+		exit(1);
 	}
 
-	if(!req->url){
+	if(!req->url)
 		req->url = url_default;
-	}
 
 	/* Init the reference timer */
-	req->ref_time = timer_tol(timer_now());
+	req->ref_time = timer_long(timer_now());
 	DBG("Reference timer = %lu\n", req->ref_time);
 
 	/* Init SSL context */
 	init_ssl();
 
-	/* Signal handling initialization  */
-	signal_init();
-
 	/* Create the master thread */
 	master = thread_make_master();
 
+	/* Signal handling initialization  */
+	signal_init();
+
 	/* Register the GET request */
 	init_sock();
 
@@ -258,12 +324,11 @@ main(int argc, char **argv)
 	 * Processing the master thread queues,
 	 * return and execute one ready thread.
 	 * Run until error, used for debuging only.
-	 * Note that not calling launch_scheduler() does
-	 * not activate SIGCHLD handling, however, this
-	 * is no issue here.
+	 * Note that not calling launch_thread_scheduler()
+	 * does not activate SIGCHLD handling, however,
+	 * this is no issue here.
 	 */
-	while (thread_fetch(master, &thread))
-		thread_call(&thread);
+	process_threads(master);
 
 	/* Finalize output informations */
 	if (req->verbose)
@@ -271,10 +336,10 @@ main(int argc, char **argv)
 			    req->url, req->response_time - req->ref_time);
 
 	/* exit cleanly */
-	FREE(url_default);
+	thread_destroy_master(master);
 	SSL_CTX_free(req->ctx);
 	free_sock(sock);
 	freeaddrinfo(req->dst);
 	FREE(req);
-	exit(0);
+	exit(exit_code);
 }
