@@ -5,8 +5,6 @@
  *
  * Part:        Socket pool utility functions.
  *
- * Version:     $Id: sock.c,v 1.1.16 2009/02/14 03:25:07 acassen Exp $
- *
  * Authors:     Alexandre Cassen, <acassen@linux-vs.org>
  *
  *              This program is distributed in the hope that it will be useful,
@@ -19,24 +17,20 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
 /* system includes */
-#include <string.h>
 
 /* keepalived includes */
 #include "memory.h"
 #include "utils.h"
-#include "list.h"
 
 /* genhash includes */
 #include "include/sock.h"
 #include "include/layer4.h"
-#include "include/ssl.h"
-#include "include/main.h"
 
 /* global var */
 SOCK *sock = NULL;
