@@ -658,7 +658,7 @@ bfd_handle_packet(bfdpkt_t *pkt)
 
 	/* Lookup session */
 	if (!pkt->hdr->remote_discr)
-		bfd = find_bfd_by_addr(&pkt->src_addr);
+		bfd = find_bfd_by_addr(&pkt->src_addr, &pkt->dst_addr);
 	else
 		bfd = find_bfd_by_discr(ntohl(pkt->hdr->remote_discr));
 
@@ -812,10 +812,11 @@ bfd_receive_packet(bfdpkt_t *pkt, int fd, char *buf, ssize_t bufsz)
 {
 	ssize_t len;
 	unsigned int ttl = 0;
-	struct msghdr msg = { 0 };
+	struct msghdr msg;
 	struct cmsghdr *cmsg = NULL;
-	char cbuf[CMSG_SPACE(sizeof (ttl))] = { 0 };
-	struct iovec iov[1] = { {0} };
+	char cbuf[CMSG_SPACE(sizeof (struct in6_pktinfo)) + CMSG_SPACE(sizeof(ttl))];
+	struct iovec iov[1];
+	struct in6_pktinfo *pktinfo;
 
 	assert(pkt);
 	assert(fd >= 0);
@@ -831,6 +832,7 @@ bfd_receive_packet(bfdpkt_t *pkt, int fd, char *buf, ssize_t bufsz)
 	msg.msg_iovlen = 1;
 	msg.msg_control = cbuf;
 	msg.msg_controllen = sizeof (cbuf);
+	msg.msg_flags = 0;	/* Unnecessary, but keep coverity happy */
 
 	len = recvmsg(fd, &msg, MSG_DONTWAIT);
 	if (len == -1) {
@@ -851,6 +853,16 @@ bfd_receive_packet(bfdpkt_t *pkt, int fd, char *buf, ssize_t bufsz)
 		if ((cmsg->cmsg_level == IPPROTO_IP && cmsg->cmsg_type == IP_TTL) ||
 		    (cmsg->cmsg_level == IPPROTO_IPV6 && cmsg->cmsg_type == IPV6_HOPLIMIT))
 			ttl = *CMSG_DATA(cmsg);
+		else if (cmsg->cmsg_level == IPPROTO_IPV6 && cmsg->cmsg_type == IPV6_PKTINFO) {
+			pktinfo = (struct in6_pktinfo *)CMSG_DATA(cmsg);
+			if (IN6_IS_ADDR_V4MAPPED(&pktinfo->ipi6_addr)) {
+				((struct sockaddr_in *)&pkt->dst_addr)->sin_addr.s_addr = pktinfo->ipi6_addr.s6_addr32[3];
+				pkt->dst_addr.ss_family = AF_INET;
+			} else {
+				memcpy(&((struct sockaddr_in6 *)&pkt->dst_addr)->sin6_addr, &pktinfo->ipi6_addr, sizeof(pktinfo->ipi6_addr));
+				pkt->dst_addr.ss_family = AF_INET6;
+			}
+		}
 		else
 			log_message(LOG_WARNING, "recvmsg() received"
 				    " unexpected control message (level %d type %d)",
@@ -931,15 +943,17 @@ bfd_open_fd_in(bfd_data_t *data)
 	hints.ai_socktype = SOCK_DGRAM;
 
 	if ((ret = getaddrinfo(NULL, BFD_CONTROL_PORT, &hints, &ai_in)))
-		log_message(LOG_ERR, "getaddrinfo() error (%s)", gai_strerror(ret));
+		log_message(LOG_ERR, "getaddrinfo() error %d (%s)", ret, gai_strerror(ret));
 	else if ((data->fd_in = socket(AF_INET6, ai_in->ai_socktype, ai_in->ai_protocol)) == -1)
-		log_message(LOG_ERR, "socket() error (%m)");
+		log_message(LOG_ERR, "socket() error %d (%m)", errno);
 	else if ((ret = setsockopt(data->fd_in, IPPROTO_IP, IP_RECVTTL, &yes, sizeof (yes))) == -1)
-		log_message(LOG_ERR, "setsockopt(IP_RECVTTL) error (%m)");
+		log_message(LOG_ERR, "setsockopt(IP_RECVTTL) error %d (%m)", errno);
 	else if ((ret = setsockopt(data->fd_in, IPPROTO_IPV6, IPV6_RECVHOPLIMIT, &yes, sizeof (yes))) == -1)
-		log_message(LOG_ERR, "setsockopt(IPV6_RECVHOPLIMIT) error (%m)");
+		log_message(LOG_ERR, "setsockopt(IPV6_RECVHOPLIMIT) error %d (%m)", errno);
+	else if ((ret = setsockopt(data->fd_in, IPPROTO_IPV6, IPV6_RECVPKTINFO, &yes, sizeof (yes))) == -1)
+		log_message(LOG_ERR, "setsockopt(IPV6_RECVPKTINFO) error %d (%m)", errno);
 	else if ((ret = bind(data->fd_in, ai_in->ai_addr, ai_in->ai_addrlen)) == -1)
-		log_message(LOG_ERR, "bind() error (%m)");
+		log_message(LOG_ERR, "bind() error %d (%m)", errno);
 
 	if (ret)
 		ret = 1;
