@@ -24,7 +24,9 @@
 
 #include <stdint.h>
 #include <stdio.h>
+#include <inttypes.h>
 
+#include "main.h"
 #include "check_data.h"
 #include "check_api.h"
 #include "check_misc.h"
@@ -49,8 +51,7 @@ check_data_t *old_check_data = NULL;
 ssl_data_t *
 alloc_ssl(void)
 {
-	ssl_data_t *ssl = (ssl_data_t *) MALLOC(sizeof(ssl_data_t));
-	return ssl;
+	return (ssl_data_t *) MALLOC(sizeof(ssl_data_t));
 }
 void
 free_ssl(void)
@@ -63,10 +64,10 @@ free_ssl(void)
 	ssl = check_data->ssl;
 
 	clear_ssl(ssl);
-	FREE_PTR(ssl->password);
-	FREE_PTR(ssl->cafile);
-	FREE_PTR(ssl->certfile);
-	FREE_PTR(ssl->keyfile);
+	FREE_CONST_PTR(ssl->password);
+	FREE_CONST_PTR(ssl->cafile);
+	FREE_CONST_PTR(ssl->certfile);
+	FREE_CONST_PTR(ssl->keyfile);
 	FREE(ssl);
 	check_data->ssl = NULL;
 }
@@ -101,9 +102,9 @@ free_vsg(void *data)
 	FREE(vsg);
 }
 static void
-dump_vsg(FILE *fp, void *data)
+dump_vsg(FILE *fp, const void *data)
 {
-	virtual_server_group_t *vsg = data;
+	const virtual_server_group_t *vsg = data;
 
 	conf_write(fp, " ------< Virtual server group >------");
 	conf_write(fp, " Virtual Server Group = %s", vsg->gname);
@@ -116,52 +117,57 @@ free_vsg_entry(void *data)
 	FREE(data);
 }
 static void
-dump_vsg_entry(FILE *fp, void *data)
+dump_vsg_entry(FILE *fp, const void *data)
 {
-	virtual_server_group_entry_t *vsg_entry = data;
+	const virtual_server_group_entry_t *vsg_entry = data;
 	uint16_t start;
 
-	if (vsg_entry->is_fwmark)
+	if (vsg_entry->is_fwmark) {
 		conf_write(fp, "   FWMARK = %u", vsg_entry->vfwmark);
-	else if (vsg_entry->range) {
-		start = vsg_entry->addr.ss_family == AF_INET ?
-			  ntohl(((struct sockaddr_in*)&vsg_entry->addr)->sin_addr.s_addr) & 0xFF :
-			  ntohs(((struct sockaddr_in6*)&vsg_entry->addr)->sin6_addr.s6_addr16[7]);
-		conf_write(fp,
-			    vsg_entry->addr.ss_family == AF_INET ?
-				"   VIP Range = %s-%d, VPORT = %d" :
-				"   VIP Range = %s-%x, VPORT = %d",
-			    inet_sockaddrtos(&vsg_entry->addr),
-			    start + vsg_entry->range,
-			    ntohs(inet_sockaddrport(&vsg_entry->addr)));
-	} else
-		conf_write(fp, "   VIP = %s, VPORT = %d"
-				    , inet_sockaddrtos(&vsg_entry->addr)
-				    , ntohs(inet_sockaddrport(&vsg_entry->addr)));
+		conf_write(fp, "   Alive: %u IPv4, %u IPv6",
+				vsg_entry->fwm4_alive, vsg_entry->fwm6_alive);
+	} else {
+		if (vsg_entry->range) {
+			start = vsg_entry->addr.ss_family == AF_INET ?
+				  ntohl(((const struct sockaddr_in*)&vsg_entry->addr)->sin_addr.s_addr) & 0xFF :
+				  ntohs(((const struct sockaddr_in6*)&vsg_entry->addr)->sin6_addr.s6_addr16[7]);
+			conf_write(fp,
+				    vsg_entry->addr.ss_family == AF_INET ?
+					"   VIP Range = %s-%u, VPORT = %d" :
+					"   VIP Range = %s-%x, VPORT = %d",
+				    inet_sockaddrtos(&vsg_entry->addr),
+				    start + vsg_entry->range,
+				    ntohs(inet_sockaddrport(&vsg_entry->addr)));
+		} else
+			conf_write(fp, "   VIP = %s, VPORT = %d"
+					    , inet_sockaddrtos(&vsg_entry->addr)
+					    , ntohs(inet_sockaddrport(&vsg_entry->addr)));
+		conf_write(fp, "     Alive: %u tcp, %u udp, %u sctp",
+			    vsg_entry->tcp_alive, vsg_entry->udp_alive, vsg_entry->sctp_alive);
+	}
+	conf_write(fp, "     reloaded = %s", vsg_entry->reloaded ? "True" : "False");
 }
 void
-alloc_vsg(char *gname)
+alloc_vsg(const char *gname)
 {
-	size_t size = strlen(gname);
 	virtual_server_group_t *new;
 
 	new = (virtual_server_group_t *) MALLOC(sizeof(virtual_server_group_t));
-	new->gname = (char *) MALLOC(size + 1);
-	memcpy(new->gname, gname, size);
+	new->gname = STRDUP(gname);
 	new->addr_range = alloc_list(free_vsg_entry, dump_vsg_entry);
 	new->vfwmark = alloc_list(free_vsg_entry, dump_vsg_entry);
 
 	list_add(check_data->vs_group, new);
 }
 void
-alloc_vsg_entry(vector_t *strvec)
+alloc_vsg_entry(const vector_t *strvec)
 {
 	virtual_server_group_t *vsg = LIST_TAIL_DATA(check_data->vs_group);
 	virtual_server_group_entry_t *new;
 	virtual_server_group_entry_t *old;
 	uint32_t start;
 	element e;
-	char *port_str;
+	const char *port_str;
 	uint32_t range;
 	unsigned fwmark;
 
@@ -169,7 +175,7 @@ alloc_vsg_entry(vector_t *strvec)
 
 	if (!strcmp(strvec_slot(strvec, 0), "fwmark")) {
 		if (!read_unsigned_strvec(strvec, 1, &fwmark, 0, UINT32_MAX, true)) {
-			report_config_error(CONFIG_GENERAL_ERROR, "(%s): fwmark '%s' must be in [0, %u] - ignoring", vsg->gname, FMT_STR_VSLOT(strvec, 1), UINT32_MAX);
+			report_config_error(CONFIG_GENERAL_ERROR, "(%s): fwmark '%s' must be in [0, %u] - ignoring", vsg->gname, strvec_slot(strvec, 1), UINT32_MAX);
 			FREE(new);
 			return;
 		}
@@ -194,14 +200,14 @@ alloc_vsg_entry(vector_t *strvec)
 			port_str = NULL;
 
 		if (inet_stosockaddr(strvec_slot(strvec, 0), port_str, &new->addr)) {
-			report_config_error(CONFIG_GENERAL_ERROR, "Invalid virtual server group IP address%s %s%s%s - skipping", FMT_STR_VSLOT(strvec, 0),
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid virtual server group IP address%s %s%s%s - skipping", strvec_slot(strvec, 0),
 						port_str ? "/port" : "", port_str ? "/" : "", port_str ? port_str : "");
 			FREE(new);
 			return;
 		}
 #ifndef LIBIPVS_USE_NL
 		if (new->addr.ss_family != AF_INET) {
-			report_config_error(CONFIG_GENERAL_ERROR, "IPVS does not support IPv6 in this build - skipping %s", FMT_STR_VSLOT(strvec, 0));
+			report_config_error(CONFIG_GENERAL_ERROR, "IPVS does not support IPv6 in this build - skipping %s", strvec_slot(strvec, 0));
 			FREE(new);
 			return;
 		}
@@ -228,7 +234,7 @@ alloc_vsg_entry(vector_t *strvec)
 				start = ntohs(((struct sockaddr_in6 *)&new->addr)->sin6_addr.s6_addr16[7]);
 
 			if (start >= new->range) {
-				report_config_error(CONFIG_GENERAL_ERROR, "Address range end is not greater than address range start - %s - skipping", FMT_STR_VSLOT(strvec, 0));
+				report_config_error(CONFIG_GENERAL_ERROR, "Address range end is not greater than address range start - %s - skipping", strvec_slot(strvec, 0));
 				FREE(new);
 				return;
 			}
@@ -245,18 +251,60 @@ static void
 free_vs(void *data)
 {
 	virtual_server_t *vs = data;
-	FREE_PTR(vs->vsgname);
-	FREE_PTR(vs->virtualhost);
+	FREE_CONST_PTR(vs->vsgname);
+	FREE_CONST_PTR(vs->virtualhost);
 	FREE_PTR(vs->s_svr);
 	free_list(&vs->rs);
 	free_notify_script(&vs->notify_quorum_up);
 	free_notify_script(&vs->notify_quorum_down);
 	FREE(vs);
 }
+
 static void
-dump_vs(FILE *fp, void *data)
+dump_forwarding_method(FILE *fp, const char *prefix, const real_server_t *rs)
 {
-	virtual_server_t *vs = data;
+	const char *fwd_method = "forwarding method = ";
+#ifdef _HAVE_IPVS_TUN_TYPE_
+	const char *csum_str = "";
+	const char *tun_type = "TUN, type = ";
+#endif
+
+	switch (rs->forwarding_method) {
+	case IP_VS_CONN_F_MASQ:
+		conf_write(fp, "   %s%sNAT", prefix, fwd_method);
+		break;
+	case IP_VS_CONN_F_DROUTE:
+		conf_write(fp, "   %s%sDR", prefix, fwd_method);
+		break;
+	case IP_VS_CONN_F_TUNNEL:
+#ifdef _HAVE_IPVS_TUN_TYPE_
+		if (rs->tun_type == IP_VS_CONN_F_TUNNEL_TYPE_IPIP)
+			conf_write(fp, "   %s%s%sIPIP", prefix, fwd_method, tun_type);
+		else {
+#ifdef _HAVE_IPVS_TUN_CSUM_
+			csum_str = rs->tun_flags == IP_VS_TUNNEL_ENCAP_FLAG_NOCSUM ? ", no checksum" :
+				   rs->tun_flags == IP_VS_TUNNEL_ENCAP_FLAG_CSUM ? ", checksum" :
+				   rs->tun_flags == IP_VS_TUNNEL_ENCAP_FLAG_REMCSUM ? ", remote checksum" :
+				   ", unknown checksum type";
+#endif
+			if (rs->tun_type == IP_VS_CONN_F_TUNNEL_TYPE_GUE)
+				conf_write(fp, "   %s%sGUE, port = %u%s", fwd_method, tun_type, ntohs(rs->tun_port), csum_str);
+#ifdef _HAVE_IPVS_TUN_GRE_
+			else if (rs->tun_type == IP_VS_CONN_F_TUNNEL_TYPE_GRE)
+				conf_write(fp, "   %s%sGRE%s", fwd_method, tun_type, csum_str);
+#endif
+		}
+#else
+		conf_write(fp, "   %s%sTUN", prefix, fwd_method);
+#endif
+		break;
+	}
+}
+
+static void
+dump_vs(FILE *fp, const void *data)
+{
+	const virtual_server_t *vs = data;
 
 	conf_write(fp, " ------< Virtual server >------");
 	if (vs->vsgname)
@@ -270,7 +318,9 @@ dump_vs(FILE *fp, void *data)
 		conf_write(fp, "   VirtualHost = %s", vs->virtualhost);
 	if (vs->af != AF_UNSPEC)
 		conf_write(fp, "   Address family = inet%s", vs->af == AF_INET ? "" : "6");
-	conf_write(fp, "   delay_loop = %lu, lvs_sched = %s", vs->delay_loop / TIMER_HZ, vs->sched);
+	conf_write(fp, "   connection timeout = %f", (double)vs->connection_to / TIMER_HZ);
+	conf_write(fp, "   delay_loop = %f", (double)vs->delay_loop / TIMER_HZ);
+	conf_write(fp, "   lvs_sched = %s", vs->sched);
 	conf_write(fp, "   Hashed = %sabled", vs->flags & IP_VS_SVC_F_HASHED ? "en" : "dis");
 #ifdef IP_VS_SVC_F_SCHED1
 	if (!strcmp(vs->sched, "sh"))
@@ -300,7 +350,7 @@ dump_vs(FILE *fp, void *data)
 		conf_write(fp, "   persistence timeout = %u", vs->persistence_timeout);
 	if (vs->persistence_granularity) {
 		if (vs->af == AF_INET6)
-			conf_write(fp, "   persistence granularity = %d",
+			conf_write(fp, "   persistence granularity = %" PRIu32,
 				       vs->persistence_granularity);
 		else
 			conf_write(fp, "   persistence granularity = %s",
@@ -316,73 +366,62 @@ dump_vs(FILE *fp, void *data)
 		conf_write(fp, "   protocol = none");
 	else
 		conf_write(fp, "   protocol = %d", vs->service_type);
-	conf_write(fp, "   alpha is %s, omega is %s",
-		    vs->alpha ? "ON" : "OFF", vs->omega ? "ON" : "OFF");
+	conf_write(fp, "   alpha is %s", vs->alpha ? "ON" : "OFF");
+	conf_write(fp, "   omega is %s", vs->omega ? "ON" : "OFF");
 	if (vs->retry != UINT_MAX)
 		conf_write(fp, "   Retry count = %u" , vs->retry);
 	if (vs->delay_before_retry != ULONG_MAX)
-		conf_write(fp, "   Retry delay = %lu" , vs->delay_before_retry / TIMER_HZ);
+		conf_write(fp, "   Retry delay = %f" , (double)vs->delay_before_retry / TIMER_HZ);
 	if (vs->warmup != ULONG_MAX)
-		conf_write(fp, "   Warmup = %lu", vs->warmup / TIMER_HZ);
+		conf_write(fp, "   Warmup = %f", (double)vs->warmup / TIMER_HZ);
 	conf_write(fp, "   Inhibit on failure is %s", vs->inhibit ? "ON" : "OFF");
 	conf_write(fp, "   quorum = %u, hysteresis = %u", vs->quorum, vs->hysteresis);
 	if (vs->notify_quorum_up)
-		conf_write(fp, "   Quorum up notify script = %s, uid:gid %d:%d",
+		conf_write(fp, "   Quorum up notify script = %s, uid:gid %u:%u",
 			    cmd_str(vs->notify_quorum_up), vs->notify_quorum_up->uid, vs->notify_quorum_up->gid);
 	if (vs->notify_quorum_down)
-		conf_write(fp, "   Quorum down notify script = %s, uid:gid %d:%d",
+		conf_write(fp, "   Quorum down notify script = %s, uid:gid %u:%u",
 			    cmd_str(vs->notify_quorum_down), vs->notify_quorum_down->uid, vs->notify_quorum_down->gid);
 	if (vs->ha_suspend)
 		conf_write(fp, "   Using HA suspend");
 	conf_write(fp, "   Using smtp notification = %s", vs->smtp_alert ? "yes" : "no");
 
-	switch (vs->forwarding_method) {
-	case IP_VS_CONN_F_MASQ:
-		conf_write(fp, "   default forwarding method = NAT");
-		break;
-	case IP_VS_CONN_F_DROUTE:
-		conf_write(fp, "   default forwarding method = DR");
-		break;
-	case IP_VS_CONN_F_TUNNEL:
-		conf_write(fp, "   default forwarding method = TUN");
-		break;
-	}
+	real_server_t rs = { .forwarding_method = vs->forwarding_method };
+#ifdef _HAVE_IPVS_TUN_TYPE_
+	rs.tun_type = vs->tun_type;
+	rs.tun_port = vs->tun_port;
+#ifdef _HAVE_IPVS_TUN_CSUM_
+	rs.tun_flags = vs->tun_flags;
+#endif
+#endif
+	dump_forwarding_method(fp, "default ", &rs);
 
 	if (vs->s_svr) {
 		conf_write(fp, "   sorry server = %s"
 				    , FMT_RS(vs->s_svr, vs));
-		switch (vs->s_svr->forwarding_method) {
-		case IP_VS_CONN_F_MASQ:
-			conf_write(fp, "   sorry server forwarding method = NAT");
-			break;
-		case IP_VS_CONN_F_DROUTE:
-			conf_write(fp, "   sorry server forwarding method = DR");
-			break;
-		case IP_VS_CONN_F_TUNNEL:
-			conf_write(fp, "   sorry server forwarding method = TUN");
-			break;
-		}
+		dump_forwarding_method(fp, "sorry server ", vs->s_svr);
 	}
+	conf_write(fp, "   alive = %d", vs->alive);
+	conf_write(fp, "   quorum_state_up = %d", vs->quorum_state_up);
+	conf_write(fp, "   reloaded = %d", vs->reloaded);
+
 	dump_list(fp, vs->rs);
 }
 
 void
-alloc_vs(char *param1, char *param2)
+alloc_vs(const char *param1, const char *param2)
 {
-	size_t size;
 	virtual_server_t *new;
-	char *port_str;
+	const char *port_str;
 	unsigned fwmark;
 
 	new = (virtual_server_t *) MALLOC(sizeof(virtual_server_t));
 
 	new->af = AF_UNSPEC;
 
-	if (!strcmp(param1, "group")) {
-		size = strlen(param2);
-		new->vsgname = (char *) MALLOC(size + 1);
-		memcpy(new->vsgname, param2, size);
-	} else if (!strcmp(param1, "fwmark")) {
+	if (!strcmp(param1, "group"))
+		new->vsgname = STRDUP(param2);
+	else if (!strcmp(param1, "fwmark")) {
 		if (!read_unsigned(param2, &fwmark, 0, UINT32_MAX, true)) {
 			report_config_error(CONFIG_GENERAL_ERROR, "virtual server fwmark '%s' must be in [0, %u] - ignoring", param2, UINT32_MAX);
 			skip_block(true);
@@ -423,6 +462,7 @@ alloc_vs(char *param1, char *param2)
 	new->quorum_state_up = true;
 	new->flags = 0;
 	new->forwarding_method = IP_VS_CONN_F_FWD_MASK;		/* So we can detect if it has been set */
+	new->connection_to = 5 * TIMER_HZ;
 	new->delay_loop = KEEPALIVED_DEFAULT_DELAY;
 	new->warmup = ULONG_MAX;
 	new->retry = UINT_MAX;
@@ -435,15 +475,26 @@ alloc_vs(char *param1, char *param2)
 
 /* Sorry server facility functions */
 void
-alloc_ssvr(char *ip, char *port)
+alloc_ssvr(const char *ip, const char *port)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
+	const char *port_str;
+
+	/* inet_stosockaddr rejects port 0 */
+	port_str = (port && port[strspn(port, "0")]) ? port : NULL;
 
 	vs->s_svr = (real_server_t *) MALLOC(sizeof(real_server_t));
 	vs->s_svr->weight = 1;
 	vs->s_svr->iweight = 1;
 	vs->s_svr->forwarding_method = vs->forwarding_method;
-	if (inet_stosockaddr(ip, port, &vs->s_svr->addr)) {
+#ifdef _HAVE_IPVS_TUN_TYPE_
+	vs->s_svr->tun_type = vs->tun_type;
+	vs->s_svr->tun_port = vs->tun_port;
+#ifdef _HAVE_IPVS_TUN_CSUM_
+	vs->s_svr->tun_flags = vs->tun_flags;
+#endif
+#endif
+	if (inet_stosockaddr(ip, port_str, &vs->s_svr->addr)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Invalid sorry server IP address %s - skipping", ip);
 		FREE(vs->s_svr);
 		vs->s_svr = NULL;
@@ -462,61 +513,76 @@ free_rs(void *data)
 #ifdef _WITH_BFD_
 	free_list(&rs->tracked_bfds);
 #endif
-	FREE_PTR(rs->virtualhost);
+	FREE_CONST_PTR(rs->virtualhost);
 	FREE(rs);
 }
 
 static void
-dump_rs(FILE *fp, void *data)
+dump_rs(FILE *fp, const void *data)
 {
-	real_server_t *rs = data;
+	const real_server_t *rs = data;
+#ifdef _WITH_BFD_
+	bfd_checker_t *cbfd;
+	element e;
+#endif
 
 	conf_write(fp, "   ------< Real server >------");
 	conf_write(fp, "   RIP = %s, RPORT = %d, WEIGHT = %d"
 			    , inet_sockaddrtos(&rs->addr)
 			    , ntohs(inet_sockaddrport(&rs->addr))
 			    , rs->weight);
-	switch (rs->forwarding_method) {
-	case IP_VS_CONN_F_MASQ:
-		conf_write(fp, "   Forwarding method = NAT");
-		break;
-	case IP_VS_CONN_F_DROUTE:
-		conf_write(fp, "   Forwarding method = DR");
-		break;
-	case IP_VS_CONN_F_TUNNEL:
-		conf_write(fp, "   Forwarding method = TUN");
-		break;
-	}
+	dump_forwarding_method(fp, "", rs);
 
 	conf_write(fp, "   Alpha is %s", rs->alpha ? "ON" : "OFF");
-	conf_write(fp, "   Delay loop = %lu" , rs->delay_loop / TIMER_HZ);
+	conf_write(fp, "   connection timeout = %f", ((double)rs->connection_to) / TIMER_HZ);
+	conf_write(fp, "   connection limit range = %" PRIu32 " -> %" PRIu32, rs->l_threshold, rs->u_threshold);
+	conf_write(fp, "   Delay loop = %f" , (double)rs->delay_loop / TIMER_HZ);
 	if (rs->retry != UINT_MAX)
 		conf_write(fp, "   Retry count = %u" , rs->retry);
 	if (rs->delay_before_retry != ULONG_MAX)
-		conf_write(fp, "   Retry delay = %lu" , rs->delay_before_retry / TIMER_HZ);
+		conf_write(fp, "   Retry delay = %f" , (double)rs->delay_before_retry / TIMER_HZ);
 	if (rs->warmup != ULONG_MAX)
-		conf_write(fp, "   Warmup = %lu", rs->warmup / TIMER_HZ);
+		conf_write(fp, "   Warmup = %f", (double)rs->warmup / TIMER_HZ);
 	conf_write(fp, "   Inhibit on failure is %s", rs->inhibit ? "ON" : "OFF");
 
 	if (rs->notify_up)
-		conf_write(fp, "     RS up notify script = %s, uid:gid %d:%d",
+		conf_write(fp, "     RS up notify script = %s, uid:gid %u:%u",
 				cmd_str(rs->notify_up), rs->notify_up->uid, rs->notify_up->gid);
 	if (rs->notify_down)
-		conf_write(fp, "     RS down notify script = %s, uid:gid %d:%d",
+		conf_write(fp, "     RS down notify script = %s, uid:gid %u:%u",
 				cmd_str(rs->notify_down), rs->notify_down->uid, rs->notify_down->gid);
 	if (rs->virtualhost)
 		conf_write(fp, "    VirtualHost = %s", rs->virtualhost);
 	conf_write(fp, "   Using smtp notification = %s", rs->smtp_alert ? "yes" : "no");
+
+	conf_write(fp, "   initial weight = %d", rs->iweight);
+	conf_write(fp, "   previous weight = %d", rs->pweight);
+	conf_write(fp, "   alive = %d", rs->alive);
+	conf_write(fp, "   num failed checkers = %u", rs->num_failed_checkers);
+	conf_write(fp, "   RS set = %d", rs->set);
+	conf_write(fp, "   reloaded = %d", rs->reloaded);
+
+#ifdef _WITH_BFD_
+	if (!LIST_ISEMPTY(rs->tracked_bfds)) {
+		conf_write(fp, "=== Tracked BFDs ===");
+		LIST_FOREACH(rs->tracked_bfds, cbfd, e)
+			conf_write(fp, "     %s", cbfd->bfd->bname);
+	}
+#endif
 }
 
 void
-alloc_rs(char *ip, char *port)
+alloc_rs(const char *ip, const char *port)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *new;
+	const char *port_str;
+
+	/* inet_stosockaddr rejects port 0 */
+	port_str = (port && port[strspn(port, "0")]) ? port : NULL;
 
 	new = (real_server_t *) MALLOC(sizeof(real_server_t));
-	if (inet_stosockaddr(ip, port, &new->addr)) {
+	if (inet_stosockaddr(ip, port_str, &new->addr)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Invalid real server ip address/port %s/%s - skipping", ip, port);
 		skip_block(true);
 		FREE(new);
@@ -543,7 +609,16 @@ alloc_rs(char *ip, char *port)
 
 	new->weight = INT_MAX;
 	new->forwarding_method = vs->forwarding_method;
+#ifdef _HAVE_IPVS_TUN_TYPE_
+	new->tun_type = vs->tun_type;
+	new->tun_port = vs->tun_port;
+#ifdef _HAVE_IPVS_TUN_CSUM_
+	new->tun_flags = vs->tun_flags;
+#endif
+#endif
 	new->alpha = -1;
+	new->inhibit = -1;
+	new->connection_to = UINT_MAX;
 	new->delay_loop = ULONG_MAX;
 	new->warmup = ULONG_MAX;
 	new->retry = UINT_MAX;
@@ -562,13 +637,13 @@ alloc_rs(char *ip, char *port)
 #ifdef _WITH_BFD_
 /* Track bfd dump */
 static void
-dump_checker_bfd(FILE *fp, void *track_data)
+dump_checker_bfd(FILE *fp, const void *track_data)
 {
-	checker_tracked_bfd_t *cbfd = track_data;
+	const checker_tracked_bfd_t *cbfd = track_data;
 
 	conf_write(fp, " Checker Track BFD = %s", cbfd->bname);
 //	conf_write(fp, "   Weight = %d", cbfd->weight);
-	conf_write(fp, "   Tracking RS = %d", cbfd->tracking_rs ? LIST_SIZE(cbfd->tracking_rs) : 0);
+	conf_write(fp, "   Tracking RS = %u", cbfd->tracking_rs ? LIST_SIZE(cbfd->tracking_rs) : 0);
 	if (cbfd->tracking_rs)
 		dump_list(fp, cbfd->tracking_rs);
 }
@@ -611,8 +686,8 @@ free_check_data(check_data_t *data)
 	FREE(data);
 }
 
-void
-dump_check_data(FILE *fp, check_data_t *data)
+static void
+dump_check_data(FILE *fp, const check_data_t *data)
 {
 	if (data->ssl) {
 		conf_write(fp, "------< SSL definitions >------");
@@ -636,8 +711,16 @@ dump_check_data(FILE *fp, check_data_t *data)
 #endif
 }
 
-char *
-format_vs (virtual_server_t *vs)
+void
+dump_data_check(FILE *fp)
+{
+	dump_global_data(fp, global_data);
+
+	dump_check_data(fp, check_data);
+}
+
+const char *
+format_vs(const virtual_server_t *vs)
 {
 	/* alloc large buffer because of unknown length of vs->vsgname */
 	static char ret[512];
@@ -655,6 +738,41 @@ format_vs (virtual_server_t *vs)
 	return ret;
 }
 
+const char *
+format_vsge(const virtual_server_group_entry_t *vsge)
+{
+	/* alloc large buffer because of unknown length of vs->vsgname */
+	static char ret[INET6_ADDRSTRLEN + 1 + 4 + 1 + 5]; /* IPv6 addr + -abcd:ppppp */
+	uint16_t start;
+
+	if (vsge->is_fwmark)
+		snprintf(ret, sizeof(ret), "FWM %u", vsge->vfwmark);
+	else if (vsge->range) {
+		start = vsge->addr.ss_family == AF_INET ?
+			  ntohl(((const struct sockaddr_in*)&vsge->addr)->sin_addr.s_addr) & 0xFF :
+			  ntohs(((const struct sockaddr_in6*)&vsge->addr)->sin6_addr.s6_addr16[7]);
+		snprintf(ret, sizeof(ret),
+			    vsge->addr.ss_family == AF_INET ?  "%s-%u,%d" : "%s-%x,%d",
+			    inet_sockaddrtos(&vsge->addr),
+			    start + vsge->range,
+			    ntohs(inet_sockaddrport(&vsge->addr)));
+	} else
+		snprintf(ret, sizeof(ret), "%s:%d",
+			    inet_sockaddrtos(&vsge->addr), ntohs(inet_sockaddrport(&vsge->addr)));
+
+	return ret;
+}
+
+const char *
+format_rs(const real_server_t *rs, const virtual_server_t *vs)
+{
+	static char buf[SOCKADDRTRIO_STR_LEN];
+
+	inet_sockaddrtotrio_r(&rs->addr, vs->service_type, buf);
+
+	return buf;
+}
+
 static void
 check_check_script_security(void)
 {
@@ -701,12 +819,13 @@ check_check_script_security(void)
 
 bool validate_check_config(void)
 {
-	element e, e1;
+	element e, e1, e2;
 	virtual_server_t *vs;
 	virtual_server_group_entry_t *vsge;
 	real_server_t *rs;
 	checker_t *checker;
 	element next;
+	unsigned weight_sum;
 
 	using_ha_suspend = false;
 	LIST_FOREACH_NEXT(check_data->vs, vs, e, next) {
@@ -716,20 +835,6 @@ bool validate_check_config(void)
 			continue;
 		}
 
-		/* Check that the quorum isn't higher than the number of real servers,
-		 * otherwise we will never be able to come up. */
-		if (vs->quorum > LIST_SIZE(vs->rs)) {
-			report_config_error(CONFIG_GENERAL_ERROR, "Warning - quorum %1$d for %2$s exceeds number of real servers %3$d, reducing quorum to %3$d", vs->quorum, FMT_VS(vs), LIST_SIZE(vs->rs));
-			vs->quorum = LIST_SIZE(vs->rs);
-		}
-
-		/* Ensure that no virtual server hysteresis >= quorum */
-		if (vs->hysteresis >= vs->quorum) {
-			report_config_error(CONFIG_GENERAL_ERROR, "Virtual server %s: hysteresis %u >= quorum %u; setting hysteresis to %u",
-					FMT_VS(vs), vs->hysteresis, vs->quorum, vs->quorum -1);
-			vs->hysteresis = vs->quorum - 1;
-		}
-
 		/* Ensure that ha_suspend is not set for any virtual server using fwmarks */
 		if (vs->ha_suspend &&
 		    (vs->vfwmark || (vs->vsg && !LIST_ISEMPTY(vs->vsg->vfwmark)))) {
@@ -763,8 +868,7 @@ bool validate_check_config(void)
 			/* Check port specified for udp/tcp/sctp unless persistent */
 			if (!vs->persistence_timeout &&
 			    !vs->vsg &&
-			    ((vs->addr.ss_family == AF_INET6 && !((struct sockaddr_in6 *)&vs->addr)->sin6_port) ||
-			     (vs->addr.ss_family == AF_INET && !((struct sockaddr_in *)&vs->addr)->sin_port))) {
+			    !inet_sockaddrport(&vs->addr)) {
 				report_config_error(CONFIG_GENERAL_ERROR, "Virtual server %s: zero port only valid for persistent services - setting", FMT_VS(vs));
 				vs->persistence_timeout = IPVS_SVC_PERSISTENT_TIMEOUT;
 			}
@@ -775,8 +879,7 @@ bool validate_check_config(void)
 		 * persistence. */
 		if (!vs->persistence_timeout && vs->vsg) {
 			LIST_FOREACH(vs->vsg->addr_range, vsge, e1) {
-				if ((vsge->addr.ss_family == AF_INET6 && !((struct sockaddr_in6 *)&vsge->addr)->sin6_port) ||
-				    (vsge->addr.ss_family == AF_INET && !((struct sockaddr_in *)&vsge->addr)->sin_port)) {
+				if (!inet_sockaddrport(&vsge->addr)) {
 					report_config_error(CONFIG_GENERAL_ERROR, "Virtual server %s: zero port only valid for persistent services - setting", FMT_VS(vs));
 					vs->persistence_timeout = IPVS_SVC_PERSISTENT_TIMEOUT;
 					break;
@@ -786,7 +889,7 @@ bool validate_check_config(void)
 
 		/* A virtual server using fwmarks will ignore any protocol setting, so warn if one is set */
 		if (vs->service_type &&
-		    ((vs->vsg && LIST_ISEMPTY(vs->vsg->addr_range) && LIST_ISEMPTY(vs->vsg->addr_range)) ||
+		    ((vs->vsg && LIST_ISEMPTY(vs->vsg->addr_range) && !LIST_ISEMPTY(vs->vsg->vfwmark)) ||
 		     (!vs->vsg && vs->vfwmark)))
 			report_config_error(CONFIG_GENERAL_ERROR, "Warning: Virtual server %s: protocol specified for fwmark - protocol will be ignored", FMT_VS(vs));
 
@@ -807,6 +910,7 @@ bool validate_check_config(void)
 		}
 
 		/* Spin through all the real servers */
+		weight_sum = 0;
 		LIST_FOREACH(vs->rs, rs, e1) {
 			/* Set the forwarding method if necessary */
 			if (rs->forwarding_method == IP_VS_CONN_F_FWD_MASK) {
@@ -815,6 +919,13 @@ bool validate_check_config(void)
 					vs->forwarding_method = IP_VS_CONN_F_MASQ;
 				}
 				rs->forwarding_method = vs->forwarding_method;
+#ifdef _HAVE_IPVS_TUN_TYPE_
+				rs->tun_type = vs->tun_type;
+				rs->tun_port = vs->tun_port;
+#ifdef _HAVE_IPVS_TUN_CSUM_
+				rs->tun_flags = vs->tun_flags;
+#endif
+#endif
 			}
 
 			/* Take default values from virtual server */
@@ -824,6 +935,8 @@ bool validate_check_config(void)
 				rs->inhibit = vs->inhibit;
 			if (rs->retry == UINT_MAX)
 				rs->retry = vs->retry;
+			if (rs->connection_to == UINT_MAX)
+				rs->connection_to = vs->connection_to;
 			if (rs->delay_loop == ULONG_MAX)
 				rs->delay_loop = vs->delay_loop;
 			if (rs->warmup == ULONG_MAX)
@@ -846,6 +959,90 @@ bool validate_check_config(void)
 					rs->smtp_alert = true;
 				}
 			}
+			weight_sum += rs->weight;
+		}
+
+		/* Check that the quorum isn't higher than the total weight of
+		 * the real servers, otherwise we will never be able to come up. */
+		if (vs->quorum > weight_sum) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Warning - quorum %u for %s exceeds total weight of real servers %u, reducing quorum to %u", vs->quorum, FMT_VS(vs), weight_sum, weight_sum);
+			vs->quorum = weight_sum;
+		}
+
+		/* Ensure that no virtual server hysteresis >= quorum */
+		if (vs->hysteresis >= vs->quorum) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Virtual server %s: hysteresis %u >= quorum %u; setting hysteresis to %u",
+					FMT_VS(vs), vs->hysteresis, vs->quorum, vs->quorum -1);
+			vs->hysteresis = vs->quorum - 1;
+		}
+
+		/* Now check that, unless using NAT, real and virtual servers have the same port.
+		 * Also if a fwmark is used, ensure that unless NAT, the real server port is 0. */
+		if (vs->vsg) {
+			if (!LIST_ISEMPTY(vs->vsg->vfwmark)) {
+				LIST_FOREACH(vs->rs, rs, e1) {
+					if (rs->forwarding_method == IP_VS_CONN_F_MASQ)
+						continue;
+					if (inet_sockaddrport(&rs->addr))
+						log_message(LOG_INFO, "WARNING - fwmark virtual server %s, real server %s has port specified - port will be ignored", FMT_VS(vs), FMT_RS(rs, vs));
+				}
+				if (vs->s_svr && vs->s_svr->forwarding_method != IP_VS_CONN_F_MASQ &&
+				    inet_sockaddrport(&vs->s_svr->addr))
+					log_message(LOG_INFO, "WARNING - fwmark virtual server %s, sorry server has port specified - port will be ignored", FMT_VS(vs));
+			}
+			LIST_FOREACH(vs->vsg->addr_range, vsge, e1) {
+				LIST_FOREACH(vs->rs, rs, e2) {
+					if (rs->forwarding_method == IP_VS_CONN_F_MASQ)
+						continue;
+					if (inet_sockaddrport(&rs->addr) &&
+					    inet_sockaddrport(&vsge->addr) != inet_sockaddrport(&rs->addr))
+						report_config_error(CONFIG_GENERAL_ERROR, "virtual server %s:[%s] and real server %s ports don't match", FMT_VS(vs), format_vsge(vsge), FMT_RS(rs, vs));
+				}
+				if (vs->s_svr && vs->s_svr->forwarding_method != IP_VS_CONN_F_MASQ &&
+				    inet_sockaddrport(&vs->s_svr->addr) &&
+				    inet_sockaddrport(&vsge->addr) != inet_sockaddrport(&vs->s_svr->addr))
+					log_message(LOG_INFO, "WARNING - virtual server %s, sorry server has port specified - port will be ignored", FMT_VS(vs));
+			}
+		} else {
+			/* We can also correct errors here */
+			LIST_FOREACH(vs->rs, rs, e1) {
+				if (rs->forwarding_method == IP_VS_CONN_F_MASQ) {
+					if (!vs->vfwmark && !inet_sockaddrport(&rs->addr))
+						inet_set_sockaddrport(&rs->addr, inet_sockaddrport(&vs->addr));
+					continue;
+				}
+
+				if (vs->vfwmark) {
+					if (inet_sockaddrport(&rs->addr)) {
+						report_config_error(CONFIG_GENERAL_ERROR, "WARNING - fwmark virtual server %s, real server %s has port specified - clearing", FMT_VS(vs), FMT_RS(rs, vs));
+						inet_set_sockaddrport(&rs->addr, 0);
+					}
+				} else {
+					if (!inet_sockaddrport(&rs->addr))
+						inet_set_sockaddrport(&rs->addr, inet_sockaddrport(&vs->addr));
+					else if (inet_sockaddrport(&vs->addr) != inet_sockaddrport(&rs->addr)) {
+						report_config_error(CONFIG_GENERAL_ERROR, "WARNING - virtual server %s and real server %s ports don't match - resetting", FMT_VS(vs), FMT_RS(rs, vs));
+						inet_set_sockaddrport(&rs->addr, inet_sockaddrport(&vs->addr));
+					}
+				}
+			}
+
+			/* Check any sorry server */
+			if (vs->s_svr && vs->s_svr->forwarding_method != IP_VS_CONN_F_MASQ) {
+				if (vs->vfwmark) {
+					if (inet_sockaddrport(&vs->s_svr->addr)) {
+						log_message(LOG_INFO, "WARNING - virtual server %s, sorry server has port specified - clearing", FMT_VS(vs));
+						inet_set_sockaddrport(&vs->s_svr->addr, 0);
+					}
+				} else {
+					if (!inet_sockaddrport(&vs->s_svr->addr))
+						inet_set_sockaddrport(&vs->s_svr->addr, inet_sockaddrport(&vs->addr));
+					else if (inet_sockaddrport(&vs->addr) != inet_sockaddrport(&vs->s_svr->addr)) {
+						report_config_error(CONFIG_GENERAL_ERROR, "WARNING - virtual server %s and sorry server ports don't match - resetting", FMT_VS(vs));
+						inet_set_sockaddrport(&vs->s_svr->addr, inet_sockaddrport(&vs->addr));
+					}
+				}
+			}
 		}
 	}
 
@@ -861,6 +1058,8 @@ bool validate_check_config(void)
 		if (checker->launch) {
 			if (checker->retry == UINT_MAX)
 				checker->retry = checker->rs->retry != UINT_MAX ? checker->rs->retry : checker->default_retry;
+			if (checker->co && checker->co->connection_to == UINT_MAX)
+				checker->co->connection_to = checker->rs->connection_to;
 			if (checker->delay_loop == ULONG_MAX)
 				checker->delay_loop = checker->rs->delay_loop;
 			if (checker->warmup == ULONG_MAX)
@@ -875,15 +1074,17 @@ bool validate_check_config(void)
 			}
 		}
 
-		/* In Alpha mode also mark the checker as failed, unless we are reloading and it has already run. */
-		if (!checker->has_run) {
+		/* In Alpha mode also mark any checker that hasn't run as failed.
+		 * Reloading is handled in migrate_checkers() */
+		if (!reload) {
 			if (checker->alpha) {
 				set_checker_state(checker, false);
 				UNSET_ALIVE(checker->rs);
-			} else {
-				/* For non alpha mode, one failure is enough initially */
-				checker->retry_it = checker->retry;
 			}
+
+			/* For non alpha mode, one failure is enough initially.
+			 * For alpha mode, log failure after one failure */
+			checker->retry_it = checker->retry;
 		}
 	}
 
