@@ -19,12 +19,15 @@
  *              2 of the License, or (at your option) any later version.
  *
  * Copyright (C) 2016 KLab Inc.
+ * Copyright (C) 2016-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
-#include <strings.h>
+#include <errno.h>
+#include <unistd.h>
 #include <stdint.h>
+#include <stdio.h>
 
 #include "check_dns.h"
 #include "check_api.h"
@@ -34,11 +37,11 @@
 #include "smtp.h"
 #include "utils.h"
 #include "parser.h"
-#include "timer.h"
 #if !HAVE_DECL_SOCK_CLOEXEC
 #include "old_socket.h"
-#include "string.h"
 #endif
+#include "layer4.h"
+#include "scheduler.h"
 
 #ifdef _DEBUG_
 #define DNS_DBG(args...) dns_log_message(thread, LOG_DEBUG, ## args)
@@ -59,9 +62,10 @@ const dns_type_t DNS_TYPE[] = {
 	{0, NULL}
 };
 
-static int dns_connect_thread(thread_t *);
+static int dns_connect_thread(thread_ref_t);
+static int dns_send_thread(thread_ref_t);
 
-static uint16_t
+static uint16_t __attribute__ ((pure))
 dns_type_lookup(const char *label)
 {
 	const dns_type_t *t;
@@ -74,7 +78,7 @@ dns_type_lookup(const char *label)
 	return 0;
 }
 
-static const char *
+static const char * __attribute__ ((pure))
 dns_type_name(uint16_t type)
 {
 	const dns_type_t *t;
@@ -87,8 +91,8 @@ dns_type_name(uint16_t type)
 	return "(unknown)";
 }
 
-static void
-dns_log_message(thread_t * thread, int level, const char *fmt, ...)
+static void __attribute__ ((format (printf, 3, 4)))
+dns_log_message(thread_ref_t thread, int level, const char *fmt, ...)
 {
 	char buf[MAX_LOG_MSG];
 	va_list args;
@@ -99,46 +103,64 @@ dns_log_message(thread_t * thread, int level, const char *fmt, ...)
 	vsnprintf(buf, sizeof (buf), fmt, args);
 	va_end(args);
 
-	log_message(level, "DNS_CHECK (%s) %s", FMT_DNS_RS(checker), buf);
+	log_message(level, "DNS_CHECK (%s) %s", FMT_CHK(checker), buf);
 }
 
-static int
-dns_final(thread_t * thread, int error, const char *fmt, ...)
+static int __attribute__ ((format (printf, 3, 4)))
+dns_final(thread_ref_t thread, int error, const char *fmt, ...)
 {
 	char buf[MAX_LOG_MSG];
 	va_list args;
+	int len;
+	bool checker_was_up;
+	bool rs_was_alive;
 
 	checker_t *checker = THREAD_ARG(thread);
 
 	DNS_DBG("final error=%d attempts=%d retry=%d", error,
 		checker->retry_it, checker->retry);
 
-	close(thread->u.fd);
+	if (thread->type != THREAD_TIMER)
+		thread_close_fd(thread);
 
 	if (error) {
-		if (checker->is_up) {
-			if (fmt) {
+		if (checker->is_up || !checker->has_run) {
+			if (fmt &&
+			    (global_data->checker_log_all_failures ||
+			     checker->log_all_failures ||
+			     checker->retry_it >= checker->retry)) {
 				va_start(args, fmt);
-				vsnprintf(buf, sizeof (buf), fmt, args);
-				dns_log_message(thread, LOG_INFO, buf);
+				len = vsnprintf(buf, sizeof (buf), fmt, args);
 				va_end(args);
+				if (checker->has_run && checker->retry_it >= checker->retry && !checker->has_run)
+					snprintf(buf + len, sizeof(buf) - len, " after %u retries", checker->retry);
+				dns_log_message(thread, LOG_INFO, "%s", buf);
 			}
 			if (checker->retry_it < checker->retry) {
 				checker->retry_it++;
+				checker->has_run = true;
 				thread_add_timer(thread->master,
 						 dns_connect_thread, checker,
 						 checker->delay_before_retry);
 				return 0;
 			}
+			checker_was_up = checker->is_up;
+			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(DOWN, checker);
-			smtp_alert(checker, NULL, NULL, "DOWN",
-				   "=> DNS_CHECK: failed on service <=");
+			if (checker_was_up && checker->rs->smtp_alert &&
+			    (rs_was_alive != checker->rs->alive || !global_data->no_checker_emails))
+				smtp_alert(SMTP_MSG_RS, checker, NULL,
+					   "=> DNS_CHECK: failed on service <=");
 		}
 	} else {
-		if (!checker->is_up) {
-			smtp_alert(checker, NULL, NULL, "UP",
-				   "=> DNS_CHECK: succeed on service <=");
+		if (!checker->is_up || !checker->has_run) {
+			checker_was_up = checker->is_up;
+			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(UP, checker);
+			if (!checker_was_up && checker->rs->smtp_alert &&
+			    (rs_was_alive != checker->rs->alive || !global_data->no_checker_emails))
+				smtp_alert(SMTP_MSG_RS, checker, NULL,
+					   "=> DNS_CHECK: succeed on service <=");
 		}
 	}
 
@@ -150,7 +172,7 @@ dns_final(thread_t * thread, int error, const char *fmt, ...)
 }
 
 static int
-dns_recv_thread(thread_t * thread)
+dns_recv_thread(thread_ref_t thread)
 {
 	unsigned long timeout;
 	ssize_t ret;
@@ -162,27 +184,27 @@ dns_recv_thread(thread_t * thread)
 	dns_check_t *dns_check = CHECKER_ARG(checker);
 
 	if (thread->type == THREAD_READ_TIMEOUT) {
-		dns_final(thread, 1, "read timeout from socket.");
+		dns_final(thread, 1, "read timeout from socket");
 		return 0;
 	}
 
 	timeout = timer_long(thread->sands) - timer_long(time_now);
 
-	ret = recv(thread->u.fd, rbuf, sizeof (rbuf), 0);
+	ret = recv(thread->u.f.fd, rbuf, sizeof (rbuf), 0);
 	if (ret == -1) {
-		if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR) {
+		if (check_EAGAIN(errno) || check_EINTR(errno)) {
 			thread_add_read(thread->master, dns_recv_thread,
-					checker, thread->u.fd, timeout);
+					checker, thread->u.f.fd, timeout, true);
 			return 0;
 		}
-		dns_final(thread, 1, "failed to read socket. %s.", strerror(errno));
+		dns_final(thread, 1, "failed to read socket. %s", strerror(errno));
 		return 0;
 	}
 
 	if (ret < (ssize_t) sizeof (r_header)) {
 		DNS_DBG("too small message. (%d bytes)", ret);
 		thread_add_read(thread->master, dns_recv_thread, checker,
-				thread->u.fd, timeout);
+				thread->u.f.fd, timeout, true);
 		return 0;
 	}
 
@@ -193,21 +215,21 @@ dns_recv_thread(thread_t * thread)
 		DNS_DBG("ID does not match. (%04x != %04x)",
 			ntohs(s_header->id), ntohs(r_header->id));
 		thread_add_read(thread->master, dns_recv_thread, checker,
-				thread->u.fd, timeout);
+				thread->u.f.fd, timeout, true);
 		return 0;
 	}
 
 	flags = ntohs(r_header->flags);
 
 	if (!DNS_QR(flags)) {
-		DNS_DBG("recieve query message?");
+		DNS_DBG("receive query message?");
 		thread_add_read(thread->master, dns_recv_thread, checker,
-				thread->u.fd, timeout);
+				thread->u.f.fd, timeout, true);
 		return 0;
 	}
 
 	if ((rcode = DNS_RC(flags)) != 0) {
-		dns_final(thread, 1, "error occurread. (rcode = %d)", rcode);
+		dns_final(thread, 1, "read error occurred. (rcode = %d)", rcode);
 		return 0;
 	}
 
@@ -223,11 +245,11 @@ dns_recv_thread(thread_t * thread)
 	} while(0)
 
 static int
-dns_make_query(thread_t * thread)
+dns_make_query(thread_ref_t thread)
 {
 	uint16_t flags = 0;
 	uint8_t *p;
-	char *s, *e;
+	const char *s, *e;
 	size_t n;
 	checker_t *checker = THREAD_ARG(thread);
 	dns_check_t *dns_check = CHECKER_ARG(checker);
@@ -235,6 +257,7 @@ dns_make_query(thread_t * thread)
 
 	DNS_SET_RD(flags, 1);	/* Recursion Desired */
 
+	/* coverity[dont_call] */
 	header->id = htons(random());
 	header->flags = htons(flags);
 	header->qdcount = htons(1);
@@ -267,51 +290,59 @@ dns_make_query(thread_t * thread)
 	return 0;
 }
 
-static int
-dns_send_thread(thread_t * thread)
+static void
+dns_send(thread_ref_t thread)
 {
+	checker_t *checker = THREAD_ARG(thread);
+	dns_check_t *dns_check = CHECKER_ARG(checker);
 	unsigned long timeout;
 	ssize_t ret;
 
-	checker_t *checker = THREAD_ARG(thread);
-	dns_check_t *dns_check = CHECKER_ARG(checker);
+	timeout = timer_long(thread->sands) - timer_long(time_now);
 
+	ret = send(thread->u.f.fd, dns_check->sbuf, dns_check->slen, 0);
+	if (ret == -1) {
+		if (check_EAGAIN(errno) || check_EINTR(errno)) {
+			thread_add_write(thread->master, dns_send_thread,
+					 checker, thread->u.f.fd, timeout, true);
+			return;
+		}
+		dns_final(thread, 1, "failed to write socket.");
+		return;
+	}
+
+	if (ret != (ssize_t) dns_check->slen) {
+		dns_final(thread, 1, "failed to write all of the datagram.");
+		return;
+	}
+
+	thread_add_read(thread->master, dns_recv_thread, checker, thread->u.f.fd, timeout, true);
+
+	return;
+}
+
+static int
+dns_send_thread(thread_ref_t thread)
+{
 	if (thread->type == THREAD_WRITE_TIMEOUT) {
 		dns_final(thread, 1, "write timeout to socket.");
 		return 0;
 	}
 
-	timeout = timer_long(thread->sands) - timer_long(time_now);
-
-	ret = send(thread->u.fd, dns_check->sbuf, dns_check->slen, 0);
-	if (ret == -1) {
-		if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR) {
-			thread_add_write(thread->master, dns_send_thread,
-					 checker, thread->u.fd, timeout);
-			return 0;
-		}
-		dns_final(thread, 1, "failed to write socket.");
-		return 0;
-	}
-
-	if (ret != (ssize_t) dns_check->slen) {
-		dns_final(thread, 1, "failed to write all of the datagram.");
-		return 0;
-	}
-
-	thread_add_read(thread->master, dns_recv_thread, checker, thread->u.fd,
-			timeout);
+	dns_send(thread);
 
 	return 0;
 }
 
 static int
-dns_check_thread(thread_t * thread)
+dns_check_thread(thread_ref_t thread)
 {
 	int status;
-	unsigned long timeout;
 
-	checker_t *checker = THREAD_ARG(thread);
+	if (thread->type == THREAD_WRITE_TIMEOUT) {
+		dns_final(thread, 1, "write timeout to socket.");
+		return 0;
+	}
 
 	status = socket_state(thread, dns_check_thread);
 
@@ -324,15 +355,18 @@ dns_check_thread(thread_t * thread)
 		dns_final(thread, 1, "connection error.");
 		break;
 	case connect_timeout:
+		dns_final(thread, 1, "connection timeout.");
+		break;
+	case connect_fail:
 		dns_final(thread, 1, "connection failure.");
 		break;
 	case connect_success:
 		dns_make_query(thread);
-		fcntl(thread->u.fd, F_SETFL,
-		      fcntl(thread->u.fd, F_GETFL, 0) | O_NONBLOCK);
-		timeout = timer_long(thread->sands) - timer_long(time_now);
-		thread_add_write(thread->master, dns_send_thread, checker,
-				 thread->u.fd, timeout);
+		dns_send(thread);
+
+		/* Cancel the write after the read is added to avoid the
+		 * file descriptor being removed */
+		thread_del_write(thread);
 		break;
 	}
 
@@ -340,9 +374,10 @@ dns_check_thread(thread_t * thread)
 }
 
 static int
-dns_connect_thread(thread_t * thread)
+dns_connect_thread(thread_ref_t thread)
 {
 	int fd, status;
+	thread_t thread_fd;
 
 	checker_t *checker = THREAD_ARG(thread);
 	conn_opts_t *co = checker->co;
@@ -353,13 +388,21 @@ dns_connect_thread(thread_t * thread)
 		return 0;
 	}
 
-	if ((fd = socket(co->dst.ss_family, SOCK_DGRAM | SOCK_CLOEXEC, IPPROTO_UDP)) == -1) {
+	if ((fd = socket(co->dst.ss_family, SOCK_DGRAM | SOCK_CLOEXEC | SOCK_NONBLOCK, IPPROTO_UDP)) == -1) {
 		dns_log_message(thread, LOG_INFO,
 				"failed to create socket. Rescheduling.");
 		thread_add_timer(thread->master, dns_connect_thread, checker,
 				 checker->delay_loop);
 		return 0;
 	}
+
+#if !HAVE_DECL_SOCK_NONBLOCK
+	if (set_sock_flags(fd, F_SETFL, O_NONBLOCK))
+		dns_log_message(thread, LOG_INFO,
+				"unable to set NONBLOCK on socket - %s (%d)",
+				strerror(errno), errno);
+#endif
+
 #if !HAVE_DECL_SOCK_CLOEXEC
 	if (set_sock_flags(fd, F_SETFD, FD_CLOEXEC))
 		dns_log_message(thread, LOG_INFO,
@@ -369,6 +412,22 @@ dns_connect_thread(thread_t * thread)
 
 	status = socket_bind_connect(fd, co);
 
+	if (status == connect_success) {
+		thread_fd = *thread;
+		thread_fd.u.f.fd = fd;
+		dns_make_query(&thread_fd);
+		dns_send(&thread_fd);
+
+		return 0;
+	}
+
+	if (status == connect_fail) {
+		close(fd);
+		dns_final(thread, 1, "network unreachable for %s", inet_sockaddrtopair(&co->dst));
+
+		return 0;
+	}
+
 	/* handle connection status & register check worker thread */
 	if (socket_connection_state(fd, status, thread, dns_check_thread, co->connection_to)) {
 		close(fd);
@@ -382,32 +441,34 @@ dns_connect_thread(thread_t * thread)
 }
 
 static void
-dns_free(void *data)
+dns_free(checker_t *checker)
 {
-	FREE(CHECKER_CO(data));
-	FREE(CHECKER_DATA(data));
-	FREE(data);
+	dns_check_t *dns_check = checker->data;
+
+	FREE_CONST(dns_check->name);
+	FREE(checker->co);
+	FREE(checker->data);
+	FREE(checker);
 }
 
 static void
-dns_dump(void *data)
+dns_dump(FILE *fp, const checker_t *checker)
 {
-	checker_t *checker = data;
-	dns_check_t *dns_check = checker->data;
+	const dns_check_t *dns_check = checker->data;
 
-	log_message(LOG_INFO, "   Keepalive method = DNS_CHECK");
-	dump_checker_opts(checker);
-	log_message(LOG_INFO, "   Type = %s", dns_type_name(dns_check->type));
-	log_message(LOG_INFO, "   Name = %s", dns_check->name);
+	conf_write(fp, "   Keepalive method = DNS_CHECK");
+	dump_checker_opts(fp, checker);
+	conf_write(fp, "   Type = %s", dns_type_name(dns_check->type));
+	conf_write(fp, "   Name = %s", dns_check->name);
 }
 
-static bool 
-dns_check_compare(void *a, void *b)
+static bool
+dns_check_compare(const checker_t *old_c, const checker_t *new_c)
 {
-	dns_check_t *old = CHECKER_DATA(a);
-	dns_check_t *new = CHECKER_DATA(b);
+	const dns_check_t *old = old_c->data;
+	const dns_check_t *new = new_c->data;
 
-	if (!compare_conn_opts(CHECKER_CO(a), CHECKER_CO(b)))
+	if (!compare_conn_opts(old_c->co, new_c->co))
 		return false;
 	if (old->type != new->type)
 		return false;
@@ -418,7 +479,7 @@ dns_check_compare(void *a, void *b)
 }
 
 static void
-dns_check_handler(__attribute__((unused)) vector_t * strvec)
+dns_check_handler(__attribute__((unused)) const vector_t *strvec)
 {
 	checker_t *checker;
 
@@ -426,7 +487,7 @@ dns_check_handler(__attribute__((unused)) vector_t * strvec)
 	dns_check->type = DNS_DEFAULT_TYPE;
 	dns_check->name = DNS_DEFAULT_NAME;
 	checker = queue_checker(dns_free, dns_dump, dns_connect_thread,
-			        dns_check_compare, dns_check, CHECKER_NEW_CO());
+				dns_check_compare, dns_check, CHECKER_NEW_CO(), true);
 
 	/* Set the non-standard retry time */
 	checker->default_retry = DNS_DEFAULT_RETRY;
@@ -434,23 +495,24 @@ dns_check_handler(__attribute__((unused)) vector_t * strvec)
 }
 
 static void
-dns_type_handler(vector_t * strvec)
+dns_type_handler(const vector_t *strvec)
 {
 	uint16_t dns_type;
 	dns_check_t *dns_check = CHECKER_GET();
 
-	dns_type = dns_type_lookup(CHECKER_VALUE_STRING(strvec));
+	dns_type = dns_type_lookup(strvec_slot(strvec, 1));
 	if (!dns_type)
-		log_message(LOG_INFO, "Unknown DNS check type %s - defaulting to SOA", vector_size(strvec) < 2 ? "[blank]" : FMT_STR_VSLOT(strvec, 1));
+		report_config_error(CONFIG_GENERAL_ERROR, "Unknown DNS check type %s - ignoring",
+				    strvec_slot(strvec, 1));
 	else
 		dns_check->type = dns_type;
 }
 
 static void
-dns_name_handler(vector_t * strvec)
+dns_name_handler(const vector_t *strvec)
 {
 	dns_check_t *dns_check = CHECKER_GET();
-	dns_check->name = CHECKER_VALUE_STRING(strvec);
+	dns_check->name = set_value(strvec);
 }
 
 static void
@@ -459,6 +521,7 @@ dns_check_end(void)
 	if (!check_conn_opts(CHECKER_GET_CO())) {
 		dequeue_new_checker();
 	}
+// Is name needed?
 }
 
 void
@@ -472,3 +535,14 @@ install_dns_check_keyword(void)
 	install_sublevel_end_handler(dns_check_end);
 	install_sublevel_end();
 }
+
+#ifdef THREAD_DUMP
+void
+register_check_dns_addresses(void)
+{
+	register_thread_address("dns_check_thread", dns_check_thread);
+	register_thread_address("dns_connect_thread", dns_connect_thread);
+	register_thread_address("dns_recv_thread", dns_recv_thread);
+	register_thread_address("dns_send_thread", dns_send_thread);
+}
+#endif
