@@ -41,37 +41,35 @@
 #include "scheduler.h"
 #endif
 
-static int tcp_connect_thread(thread_t *);
+static int tcp_connect_thread(thread_ref_t);
 
 /* Configuration stream handling */
 static void
-free_tcp_check(void *data)
+free_tcp_check(checker_t *checker)
 {
-	FREE(CHECKER_CO(data));
-	FREE(data);
+	FREE(checker->co);
+	FREE(checker);
 }
 
 static void
-dump_tcp_check(FILE *fp, void *data)
+dump_tcp_check(FILE *fp, const checker_t *checker)
 {
-	checker_t *checker = data;
-
 	conf_write(fp, "   Keepalive method = TCP_CHECK");
 	dump_checker_opts(fp, checker);
 }
 
 static bool
-tcp_check_compare(void *a, void *b)
+tcp_check_compare(const checker_t *old_c, const checker_t *new_c)
 {
-	return compare_conn_opts(CHECKER_CO(a), CHECKER_CO(b));
+	return compare_conn_opts(old_c->co, new_c->co);
 }
 
 static void
-tcp_check_handler(__attribute__((unused)) vector_t *strvec)
+tcp_check_handler(__attribute__((unused)) const vector_t *strvec)
 {
 	/* queue new checker */
 	queue_checker(free_tcp_check, dump_tcp_check, tcp_connect_thread,
-		      tcp_check_compare, NULL, CHECKER_NEW_CO());
+		      tcp_check_compare, NULL, CHECKER_NEW_CO(), true);
 }
 
 static void
@@ -93,7 +91,7 @@ install_tcp_check_keyword(void)
 }
 
 static void
-tcp_epilog(thread_t * thread, bool is_success)
+tcp_epilog(thread_ref_t thread, bool is_success)
 {
 	checker_t *checker;
 	unsigned long delay;
@@ -108,7 +106,7 @@ tcp_epilog(thread_t * thread, bool is_success)
 
 		if (is_success && (!checker->is_up || !checker->has_run)) {
 			log_message(LOG_INFO, "TCP connection to %s success."
-					, FMT_TCP_RS(checker));
+					, FMT_CHK(checker));
 			checker_was_up = checker->is_up;
 			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(UP, checker);
@@ -118,15 +116,15 @@ tcp_epilog(thread_t * thread, bool is_success)
 					   "=> TCP CHECK succeed on service <=");
 		} else if (!is_success && 
 			   (checker->is_up || !checker->has_run)) {
-			if (checker->retry)
+			if (checker->retry && checker->has_run)
 				log_message(LOG_INFO
-				    , "Check on service %s failed after %d retries."
-				    , FMT_TCP_RS(checker)
+				    , "TCP_CHECK on service %s failed after %u retries."
+				    , FMT_CHK(checker)
 				    , checker->retry);
 			else
 				log_message(LOG_INFO
-				    , "Check on service %s failed."
-				    , FMT_TCP_RS(checker));
+				    , "TCP_CHECK on service %s failed."
+				    , FMT_CHK(checker));
 			checker_was_up = checker->is_up;
 			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(DOWN, checker);
@@ -140,12 +138,14 @@ tcp_epilog(thread_t * thread, bool is_success)
 		++checker->retry_it;
 	}
 
+	checker->has_run = true;
+
 	/* Register next timer checker */
 	thread_add_timer(thread->master, tcp_connect_thread, checker, delay);
 }
 
 static int
-tcp_check_thread(thread_t * thread)
+tcp_check_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	int status;
@@ -164,15 +164,17 @@ tcp_check_thread(thread_t * thread)
 		tcp_epilog(thread, true);
 		break;
 	case connect_timeout:
-		if (checker->is_up)
-			log_message(LOG_INFO, "TCP connection to %s timeout."
-					, FMT_TCP_RS(checker));
+		if (checker->is_up &&
+		    (global_data->checker_log_all_failures || checker->log_all_failures))
+			log_message(LOG_INFO, "TCP connection to %s timedout."
+					, FMT_CHK(checker));
 		tcp_epilog(thread, false);
 		break;
 	default:
-		if (checker->is_up)
+		if (checker->is_up &&
+		    (global_data->checker_log_all_failures || checker->log_all_failures))
 			log_message(LOG_INFO, "TCP connection to %s failed."
-					, FMT_TCP_RS(checker));
+					, FMT_CHK(checker));
 		tcp_epilog(thread, false);
 	}
 
@@ -180,7 +182,7 @@ tcp_check_thread(thread_t * thread)
 }
 
 static int
-tcp_connect_thread(thread_t * thread)
+tcp_connect_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	conn_opts_t *co = checker->co;
@@ -221,9 +223,14 @@ tcp_connect_thread(thread_t * thread)
 	if(tcp_connection_state(fd, status, thread, tcp_check_thread,
 			co->connection_to)) {
 		close(fd);
-		log_message(LOG_INFO, "TCP socket bind failed. Rescheduling.");
-		thread_add_timer(thread->master, tcp_connect_thread, checker,
-				checker->delay_loop);
+
+		if (status == connect_fail) {
+			tcp_epilog(thread, false);
+		} else {
+			log_message(LOG_INFO, "TCP socket bind failed. Rescheduling.");
+			thread_add_timer(thread->master, tcp_connect_thread, checker,
+					checker->delay_loop);
+		}
 	}
 
 	return 0;
