@@ -111,6 +111,8 @@ set_vrrp_defaults(data_t * data)
 	strcpy(data->vrrp_ipset_address, DEFAULT_IPSET_NAME);
 	strcpy(data->vrrp_ipset_address6, DEFAULT_IPSET_NAME "6");
 	strcpy(data->vrrp_ipset_address_iface6, DEFAULT_IPSET_NAME "if6");
+	strcpy(data->vrrp_ipset_igmp, DEFAULT_IPSET_NAME "_igmp");
+	strcpy(data->vrrp_ipset_mld, DEFAULT_IPSET_NAME "_mld");
 #endif
 	data->vrrp_check_unicast_src = false;
 	data->vrrp_skip_check_adv_addr = false;
@@ -531,6 +533,10 @@ dump_global_data(FILE *fp, data_t * data)
 				conf_write(fp," ipset IPv6 address set = %s", data->vrrp_ipset_address6);
 			if (data->vrrp_ipset_address_iface6[0])
 				conf_write(fp," ipset IPv6 address,iface set = %s", data->vrrp_ipset_address_iface6);
+			if (data->vrrp_ipset_igmp[0])
+				conf_write(fp," ipset IGMP set = %s", data->vrrp_ipset_igmp);
+			if (data->vrrp_ipset_mld[0])
+				conf_write(fp," ipset MLD set = %s", data->vrrp_ipset_mld);
 		}
 #endif
 	}
@@ -540,7 +546,12 @@ dump_global_data(FILE *fp, data_t * data)
 		conf_write(fp," nftables table name = %s", data->vrrp_nf_table_name);
 		conf_write(fp," nftables base chain priority = %d", data->vrrp_nf_chain_priority);
 		conf_write(fp," nftables with%s counters", data->vrrp_nf_counters ? "" : "out");
-		conf_write(fp," nftables %suse ifname for link local IPv6", data->vrrp_nf_ifindex ? "don't " : "");
+		conf_write(fp," nftables %sforce use ifindex for link local IPv6", data->vrrp_nf_ifindex ? "" : "don't ");
+		if (data->nft_version)
+			conf_write(fp," nft version %u.%u.%u", data->nft_version >> 16,
+					(data->nft_version >> 8) & 0xff, data->nft_version & 0xff);
+		conf_write(fp," libnftnl version %u.%u.%u", LIBNFTNL_VERSION >> 16,
+			       (LIBNFTNL_VERSION >> 8) & 0xff, LIBNFTNL_VERSION & 0xff);
 	}
 #endif
 
