@@ -888,6 +888,32 @@ vrrp_ipsets_handler(const vector_t *strvec)
 		global_data->vrrp_ipset_address_iface6[sizeof(global_data->vrrp_ipset_address_iface6) - 5] = '\0';
 		strcat(global_data->vrrp_ipset_address_iface6, "_if6");
 	}
+	if (vector_size(strvec) >= 5) {
+		if (strlen(strvec_slot(strvec,4)) >= sizeof(global_data->vrrp_ipset_igmp)-1) {
+			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset IGMP name too long - ignored");
+			return;
+		}
+		strcpy(global_data->vrrp_ipset_igmp, strvec_slot(strvec,4));
+	}
+	else {
+		/* No second set specified, copy first name and add "_igmp" */
+		strcpy(global_data->vrrp_ipset_igmp, global_data->vrrp_ipset_address);
+		global_data->vrrp_ipset_address6[sizeof(global_data->vrrp_ipset_igmp) - 6] = '\0';
+		strcat(global_data->vrrp_ipset_igmp, "_igmp");
+	}
+	if (vector_size(strvec) >= 6) {
+		if (strlen(strvec_slot(strvec,5)) >= sizeof(global_data->vrrp_ipset_mld)-1) {
+			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset MLD name too long - ignored");
+			return;
+		}
+		strcpy(global_data->vrrp_ipset_mld, strvec_slot(strvec,5));
+	}
+	else {
+		/* No second set specified, copy first name and add "_mld" */
+		strcpy(global_data->vrrp_ipset_mld, global_data->vrrp_ipset_address);
+		global_data->vrrp_ipset_mld[sizeof(global_data->vrrp_ipset_mld) - 5] = '\0';
+		strcat(global_data->vrrp_ipset_mld, "_mld");
+	}
 }
 #endif
 #endif
