@@ -18,14 +18,17 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
+#include <errno.h>
+#include <unistd.h>
+
 #include "layer4.h"
-#include "utils.h"
 #include "logger.h"
+#include "scheduler.h"
 
 #ifndef _WITH_LVS_
 static
@@ -38,7 +41,6 @@ socket_bind_connect(int fd, conn_opts_t *co)
 	struct linger li;
 	socklen_t addrlen;
 	int ret;
-	int val;
 	struct sockaddr_storage *addr = &co->dst;
 	struct sockaddr_storage *bind_addr = &co->bindto;
 
@@ -48,20 +50,18 @@ socket_bind_connect(int fd, conn_opts_t *co)
 		return connect_error;
 	}
 	if (opt == SOCK_STREAM) {
-		/* free the tcp port after closing the socket descriptor */
+		/* free the tcp port after closing the socket descriptor, but
+		 * allow time for a proper shutdown. */
 		li.l_onoff = 1;
-		li.l_linger = 0;
-		setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *) &li, sizeof (struct linger));
+		li.l_linger = 5;
+		if (setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *) &li, sizeof (struct linger)))
+			log_message(LOG_INFO, "Failed to set SO_LINGER for socket %d - errno %d (%m)", fd, errno);
 	}
 
-	/* Make socket non-block. */
-	val = fcntl(fd, F_GETFL, 0);
-	fcntl(fd, F_SETFL, val | O_NONBLOCK);
-
 #ifdef _WITH_SO_MARK_
 	if (co->fwmark) {
 		if (setsockopt (fd, SOL_SOCKET, SO_MARK, &co->fwmark, sizeof (co->fwmark)) < 0) {
-			log_message(LOG_ERR, "Error setting fwmark %d to socket: %s", co->fwmark, strerror(errno));
+			log_message(LOG_ERR, "Error setting fwmark %u to socket: %s", co->fwmark, strerror(errno));
 			return connect_error;
 		}
 	}
@@ -88,55 +88,52 @@ socket_bind_connect(int fd, conn_opts_t *co)
 	ret = connect(fd, (struct sockaddr *) addr, addrlen);
 
 	/* Immediate success */
-	if (ret == 0) {
-		fcntl(fd, F_SETFL, val);
+	if (ret == 0)
 		return connect_success;
-	}
 
 	/* If connect is in progress then return 1 else it's real error. */
-	if (ret < 0) {
-		if (errno != EINPROGRESS) {
-/*			log_message(LOG_INFO, "Checker connect failed: %s", strerror(errno)); */
-			return connect_error;
-		}
-	}
+	if (errno == EINPROGRESS)
+		return connect_in_progress;
 
-	/* restore previous fd args */
-	fcntl(fd, F_SETFL, val);
-	return connect_in_progress;
+	/* ENETUNREACH can be returned here. I'm not sure
+	 * about any of the others, but play safe. These
+	 * should all be considered to be a failure to connect
+	 * rather than a failure to run the check. */
+	if (errno == ENETUNREACH || errno == EHOSTUNREACH ||
+	    errno == ECONNREFUSED || errno == EHOSTDOWN ||
+	    errno == ENETDOWN || errno == ECONNRESET ||
+	    errno == ECONNABORTED || errno == ETIMEDOUT)
+		return connect_fail;
+
+	return connect_error;
 }
 
 enum connect_result
 socket_connect(int fd, struct sockaddr_storage *addr)
 {
-	conn_opts_t co;
-	memset(&co, 0, sizeof(co));
-	co.dst = *addr;
+	conn_opts_t co = { .dst = *addr };
+
 	return socket_bind_connect(fd, &co);
 }
 
 enum connect_result
-socket_state(thread_t * thread, int (*func) (thread_t *))
+socket_state(thread_ref_t thread, thread_func_t func)
 {
 	int status;
 	socklen_t addrlen;
-	int ret = 0;
 	timeval_t timer_min;
 
 	/* Handle connection timeout */
 	if (thread->type == THREAD_WRITE_TIMEOUT) {
-		close(thread->u.fd);
+		thread_close_fd(thread);
 		return connect_timeout;
 	}
 
 	/* Check file descriptor */
 	addrlen = sizeof(status);
-	if (getsockopt(thread->u.fd, SOL_SOCKET, SO_ERROR, (void *) &status, &addrlen) < 0)
-		ret = errno;
-
-	/* Connection failed !!! */
-	if (ret) {
-		close(thread->u.fd);
+	if (getsockopt(thread->u.f.fd, SOL_SOCKET, SO_ERROR, (void *) &status, &addrlen) < 0) {
+		/* getsockopt failed !!! */
+		thread_close_fd(thread);
 		return connect_error;
 	}
 
@@ -145,40 +142,41 @@ socket_state(thread_t * thread, int (*func) (thread_t *))
 	 * and other error code until connection is established.
 	 * Recompute the write timeout (or pending connection).
 	 */
+	if (status == 0)
+		return connect_success;
+
 	if (status == EINPROGRESS) {
 		timer_min = timer_sub_now(thread->sands);
 		thread_add_write(thread->master, func, THREAD_ARG(thread),
-				 thread->u.fd, -timer_long(timer_min));
+				 thread->u.f.fd, -timer_long(timer_min), true);
 		return connect_in_progress;
-	} else if (status != 0) {
-		close(thread->u.fd);
-		return connect_error;
 	}
 
-	return connect_success;
+	thread_close_fd(thread);
+
+	if (status == ETIMEDOUT)
+		return connect_timeout;
+
+	/* Since the connect() call succeeded, treat this as a
+	 * failure to establish a connection. */
+	return connect_fail;
 }
 
 #ifdef _WITH_LVS_
-int
-socket_connection_state(int fd, enum connect_result status, thread_t * thread,
-		     int (*func) (thread_t *), unsigned long timeout)
+bool
+socket_connection_state(int fd, enum connect_result status, thread_ref_t thread,
+		     thread_func_t func, unsigned long timeout)
 {
 	void *checker;
 
 	checker = THREAD_ARG(thread);
 
-	switch (status) {
-	case connect_success:
-		thread_add_write(thread->master, func, checker, fd, timeout);
-		return 0;
-
-		/* Checking non-blocking connect, we wait until socket is writable */
-	case connect_in_progress:
-		thread_add_write(thread->master, func, checker, fd, timeout);
-		return 0;
-
-	default:
-		return 1;
+	if (status == connect_success ||
+	    status == connect_in_progress) {
+		thread_add_write(thread->master, func, checker, fd, timeout, true);
+		return false;
 	}
+
+	return true;
 }
 #endif
