@@ -40,6 +40,7 @@
 #include <getopt.h>
 #include <linux/version.h>
 #include <ctype.h>
+#include <stdlib.h>
 
 #include "main.h"
 #include "global_data.h"
@@ -68,6 +69,9 @@
 #ifdef _WITH_JSON_
 #include "vrrp_json.h"
 #endif
+#ifdef _WITH_NFTABLES_
+#include "vrrp_nftables.h"
+#endif
 #endif
 #ifdef _WITH_BFD_
 #include "bfd_daemon.h"
@@ -871,6 +875,18 @@ signal_init(void)
 	signal_ignore(SIGPIPE);
 }
 
+static void
+signals_ignore(void) {
+#ifndef _DEBUG_
+	signal_ignore(SIGHUP);
+	signal_ignore(SIGUSR1);
+	signal_ignore(SIGUSR2);
+#ifdef _WITH_JSON_
+	signal_ignore(SIGJSON);
+#endif
+#endif
+}
+
 /* To create a core file when abrt is running (a RedHat distribution),
  * and keepalived isn't installed from an RPM package, edit the file
  * “/etc/abrt/abrt.conf”, and change the value of the field
@@ -1732,6 +1748,15 @@ keepalived_main(int argc, char **argv)
 	struct utsname uname_buf;
 	char *end;
 	int exit_code = KEEPALIVED_EXIT_OK;
+#ifdef _WITH_NFTABLES_
+	FILE *fp;
+	char nft_ver_buf[128];
+	char *p;
+	unsigned nft_major = 0, nft_minor = 0, nft_release = 0;
+#endif
+
+	/* Ignore reloading signals till signal_init call */
+	signals_ignore();
 
 	/* Ensure time_now is set. We then don't have to check anywhere
 	 * else if it is set. */
@@ -1862,6 +1887,26 @@ keepalived_main(int argc, char **argv)
 
 	init_global_data(global_data, NULL, false);
 
+#ifdef _WITH_NFTABLES_
+	if (global_data->vrrp_nf_table_name) {
+		/* We are using nftables. Find the version of nft. */
+		fp = popen("nft -v 2>/dev/null", "r");
+		if (fgets(nft_ver_buf, sizeof(nft_ver_buf) - 1, fp)) {
+			p = strchr(nft_ver_buf, ' ');
+			while (*p == ' ')
+				p++;
+			if (*p == 'v')
+				p++;
+
+			if (sscanf(p, "%u.%u.%u", &nft_major, &nft_minor, &nft_release) >= 2)
+				global_data->nft_version = (nft_major * 0x100 + nft_minor) * 0x100 + nft_release;
+		}
+		pclose(fp);
+
+		set_nf_ifname_type();
+	}
+#endif
+
 	/* Update process name if necessary */
 	if (global_data->process_name)
 		set_process_name(global_data->process_name);
