@@ -49,7 +49,7 @@ extern void free_bfd_data(bfd_data_t *);
 extern void bfd_complete_init(void);
 extern void alloc_bfd_buffer(void);
 extern void free_bfd_buffer(void);
-extern bfd_t *find_bfd_by_addr(const struct sockaddr_storage *) __attribute__ ((pure));
+extern bfd_t *find_bfd_by_addr(const struct sockaddr_storage *, const struct sockaddr_storage *) __attribute__ ((pure));
 extern bfd_t *find_bfd_by_discr(const uint32_t) __attribute__ ((pure));
 extern bfd_t *find_bfd_by_name(const char *) __attribute__ ((pure));
 extern uint32_t rand_intv(uint32_t, uint32_t);
