@@ -54,6 +54,13 @@ struct ip_vs_dest_app {
 	struct ip_vs_dest_user	user;
 	uint16_t		af;
 	union nf_inet_addr	nf_addr;
+#ifdef _HAVE_IPVS_TUN_TYPE_
+	int			tun_type;
+	int			tun_port;
+#ifdef _HAVE_IPVS_TUN_CSUM_
+	int			tun_flags;
+#endif
+#endif
 };
 
 
