@@ -38,10 +38,19 @@
 #define DEFAULT_IPTABLES_CHAIN_IN	"INPUT"
 #define DEFAULT_IPTABLES_CHAIN_OUT	"OUTPUT"
 
+typedef enum {
+        NOT_INIT,
+        INIT_SUCCESS,
+        INIT_FAILED,
+} init_state_t;
+
 /* prototypes */
 extern void handle_iptable_rule_to_iplist(list, list, int, bool force);
 extern void handle_iptables_accept_mode(vrrp_t *, int, bool);
-extern void iptables_init(void);
+#ifdef _HAVE_VRRP_VMAC_
+extern void iptables_add_vmac(const vrrp_t *);
+extern void iptables_remove_vmac(const vrrp_t *);
+#endif
 extern void iptables_startup(bool);
 extern void iptables_cleanup(void);
 extern void iptables_fini(void);
