@@ -26,10 +26,15 @@
 #include "config.h"
 
 #include "vrrp_ipaddress.h"
+#include "vrrp_iptables.h"
 
 /* prototypes */
 extern void handle_iptable_rule_to_vip_cmd(ip_address_t *, int, bool);
-extern void check_chains_exist_cmd(void);
-extern void iptables_init_cmd(void);
+#ifdef _HAVE_VRRP_VMAC_
+extern void handle_iptable_rule_for_igmp_cmd(const char *, int, uint8_t);
+#endif
+extern init_state_t check_chains_exist_cmd(uint8_t);
+extern init_state_t iptables_init_cmd(uint8_t);
+extern void iptables_fini_cmd(void);
 
 #endif
