@@ -105,6 +105,16 @@ typedef struct dbus_queue_ent {
 	GVariant *args;
 } dbus_queue_ent_t;
 
+#define DBUS_SERVICE_NAME			"org.keepalived.Vrrp1"
+#define DBUS_VRRP_INTERFACE			"org.keepalived.Vrrp1.Vrrp"
+#define DBUS_VRRP_OBJECT_ROOT			"/org/keepalived/Vrrp1"
+#define DBUS_VRRP_INSTANCE_PATH_DEFAULT_LENGTH	8
+#define DBUS_VRRP_INSTANCE_INTERFACE		"org.keepalived.Vrrp1.Instance"
+#define DBUS_VRRP_INTERFACE_FILE_PATH		"/usr/share/dbus-1/interfaces/org.keepalived.Vrrp1.Vrrp.xml"
+#define DBUS_VRRP_INSTANCE_INTERFACE_FILE_PATH	"/usr/share/dbus-1/interfaces/org.keepalived.Vrrp1.Instance.xml"
+
+static bool dbus_running;
+
 /* Global file variables */
 static GDBusNodeInfo *vrrp_introspection_data = NULL;
 static GDBusNodeInfo *vrrp_instance_introspection_data = NULL;
@@ -114,7 +124,8 @@ static GMainLoop *loop;
 
 /* Data passing between main vrrp thread and dbus thread */
 dbus_queue_ent_t *ent_ptr;
-static int dbus_in_pipe[2], dbus_out_pipe[2];
+static int dbus_in_pipe[2] = {-1, -1};
+static int dbus_out_pipe[2] = {-1, -1};
 static sem_t thread_end;
 
 /* The only characters that are valid in a dbus path are A-Z, a-z, 0-9, _ */
@@ -135,7 +146,7 @@ set_valid_path(char *valid_path, const char *path)
 	return valid_path;
 }
 
-static bool
+static bool __attribute__ ((pure))
 valid_path_cmp(const char *path, const char *valid_path)
 {
 	for ( ; *path && *valid_path; path++, valid_path++) {
@@ -176,7 +187,7 @@ state_str(int state)
 	return "Unknown";
 }
 
-static vrrp_t *
+static vrrp_t * __attribute__ ((pure))
 get_vrrp_instance(const char *ifname, int vrid, int family)
 {
 	element e;
@@ -191,21 +202,27 @@ get_vrrp_instance(const char *ifname, int vrid, int family)
 		if (vrrp->vrid == vrid &&
 		    vrrp->family == family &&
 		    !valid_path_cmp(IF_BASE_IFP(vrrp->ifp)->ifname, ifname))
-				return vrrp;
+			return vrrp;
 	}
 
 	return NULL;
 }
 
 static gboolean
-unregister_object(gpointer key, gpointer value, __attribute__((unused)) gpointer user_data)
+unregister_object(const void * const key, gpointer value, __attribute__((unused)) gpointer user_data)
 {
 	if (g_hash_table_remove(objects, key))
 		return g_dbus_connection_unregister_object(global_connection, GPOINTER_TO_UINT(value));
 	return false;
 }
 
-static gchar *
+static gboolean
+remove_object(__attribute__((unused)) gpointer key, gpointer value, __attribute__((unused)) gpointer user_data)
+{
+	return g_dbus_connection_unregister_object(global_connection, GPOINTER_TO_UINT(value));
+}
+
+static gchar * __attribute__ ((malloc))
 dbus_object_create_path_vrrp(void)
 {
 	return g_strconcat(DBUS_VRRP_OBJECT_ROOT,
@@ -217,7 +234,7 @@ dbus_object_create_path_vrrp(void)
 			  "/Vrrp", NULL);
 }
 
-static gchar *
+static gchar * __attribute__ ((malloc))
 dbus_object_create_path_instance(const gchar *interface, int vrid, sa_family_t family)
 {
 	gchar *object_path;
@@ -245,19 +262,19 @@ static dbus_queue_ent_t *
 process_method_call(dbus_queue_ent_t *ent)
 {
 	ssize_t ret;
+	char buf = 0;
 
 	if (!ent)
 		return NULL;
 
 	/* Tell the main thread that a queue entry is waiting. Any data works */
 	ent_ptr = ent;
-	if (write(dbus_in_pipe[1], ent, 1) != 1)
+	if (write(dbus_in_pipe[1], &buf, 1) != 1)
 		log_message(LOG_INFO, "Write from DBus thread to main thread failed");
 
 	/* Wait for a response */
-	while ((ret = read(dbus_out_pipe[0], ent, 1)) == -1 && errno == EINTR) {
+	while ((ret = read(dbus_out_pipe[0], &buf, 1)) == -1 && check_EINTR(errno))
 		log_message(LOG_INFO, "dbus_out_pipe read returned EINTR");
-	}
 	if (ret == -1)
 		log_message(LOG_INFO, "DBus response read error - errno = %d", errno);
 
@@ -459,7 +476,7 @@ static const GDBusInterfaceVTable interface_vtable =
 };
 
 static int
-dbus_create_object_params(char *instance_name, const char *interface_name, int vrid, sa_family_t family, bool log_success)
+dbus_create_object_params(const char *instance_name, const char *interface_name, int vrid, sa_family_t family, bool log_success)
 {
 	gchar *object_path;
 	GError *local_error = NULL;
@@ -481,7 +498,7 @@ dbus_create_object_params(char *instance_name, const char *interface_name, int v
 	}
 
 	if (instance) {
-		g_hash_table_insert(objects, instance_name, GUINT_TO_POINTER(instance));
+		g_hash_table_insert(objects, no_const_char_p(instance_name), GUINT_TO_POINTER(instance));
 		if (log_success)
 			log_message(LOG_INFO, "Added DBus object for instance %s on path %s", instance_name, object_path);
 	}
@@ -526,32 +543,32 @@ on_bus_acquired(GDBusConnection *connection,
 {
 	global_connection = connection;
 	gchar *path;
+	vrrp_t *vrrp;
 	element e;
 	GError *local_error = NULL;
+	guint vrrp_guint;
 
 	log_message(LOG_INFO, "Acquired DBus bus %s", name);
 
 	/* register VRRP object */
 	path = dbus_object_create_path_vrrp();
-	guint vrrp = g_dbus_connection_register_object(connection, path,
+	vrrp_guint = g_dbus_connection_register_object(connection, path,
 							 vrrp_introspection_data->interfaces[0],
 							 &interface_vtable, NULL, NULL, &local_error);
-	g_hash_table_insert(objects, "__Vrrp__", GUINT_TO_POINTER(vrrp));
-	g_free(path);
+	g_hash_table_insert(objects, no_const_char_p("__Vrrp__"), GUINT_TO_POINTER(vrrp_guint));
 	if (local_error != NULL) {
 		log_message(LOG_INFO, "Registering VRRP object on %s failed: %s",
 			    path, local_error->message);
 		g_clear_error(&local_error);
 	}
+	g_free(path);
 
 	/* for each available VRRP instance, register an object */
 	if (LIST_ISEMPTY(vrrp_data->vrrp))
 		return;
 
-	for (e = LIST_HEAD(vrrp_data->vrrp); e; ELEMENT_NEXT(e)) {
-		vrrp_t * vrrp = ELEMENT_DATA(e);
+	LIST_FOREACH(vrrp_data->vrrp, vrrp, e)
 		dbus_create_object(vrrp);
-	}
 
 	/* Send a signal to say we have started */
 	path = dbus_object_create_path_vrrp();
@@ -559,10 +576,8 @@ on_bus_acquired(GDBusConnection *connection,
 	g_free(path);
 
 	/* Notify DBus of the state of our instances */
-	for (e = LIST_HEAD(vrrp_data->vrrp); e; ELEMENT_NEXT(e)) {
-		vrrp_t * vrrp = ELEMENT_DATA(e);
+	LIST_FOREACH(vrrp_data->vrrp, vrrp, e)
 		dbus_send_state_signal(vrrp);
-	}
 }
 
 /* run if bus name is acquired successfully */
@@ -582,22 +597,26 @@ on_name_lost(GDBusConnection *connection,
 {
 	log_message(LOG_INFO, "Lost the name %s on the session bus", name);
 	global_connection = connection;
-	g_hash_table_foreach_remove(objects, unregister_object, NULL);
+	g_hash_table_foreach_remove(objects, remove_object, NULL);
 	objects = NULL;
 	global_connection = NULL;
 }
 
-static gchar*
-read_file(gchar* filepath)
+static const gchar*
+read_file(const gchar* filepath)
 {
 	FILE * f;
-	size_t length;
+	long length;
 	gchar *ret = NULL;
 
 	f = fopen(filepath, "r");
 	if (f) {
 		fseek(f, 0, SEEK_END);
-		length = (size_t)ftell(f);
+		length = ftell(f);
+		if (length < 0) {
+			fclose(f);
+			return NULL;
+		}
 		fseek(f, 0, SEEK_SET);
 
 		/* We can't use MALLOC since it isn't thread safe */
@@ -619,7 +638,7 @@ read_file(gchar* filepath)
 static void *
 dbus_main(__attribute__ ((unused)) void *unused)
 {
-	gchar *introspection_xml;
+	const gchar *introspection_xml;
 	guint owner_id;
 	const char *service_name;
 
@@ -641,7 +660,7 @@ dbus_main(__attribute__ ((unused)) void *unused)
 	if (!introspection_xml)
 		return NULL;
 	vrrp_introspection_data = g_dbus_node_info_new_for_xml(introspection_xml, &error);
-	FREE(introspection_xml);
+	FREE_CONST(introspection_xml);
 	if (error != NULL) {
 		log_message(LOG_INFO, "Parsing DBus interface %s from file %s failed: %s",
 			    DBUS_VRRP_INTERFACE, DBUS_VRRP_INTERFACE_FILE_PATH, error->message);
@@ -653,7 +672,7 @@ dbus_main(__attribute__ ((unused)) void *unused)
 	if (!introspection_xml)
 		return NULL;
 	vrrp_instance_introspection_data = g_dbus_node_info_new_for_xml(introspection_xml, &error);
-	FREE(introspection_xml);
+	FREE_CONST(introspection_xml);
 	if (error != NULL) {
 		log_message(LOG_INFO, "Parsing DBus interface %s from file %s failed: %s",
 			    DBUS_VRRP_INSTANCE_INTERFACE, DBUS_VRRP_INSTANCE_INTERFACE_FILE_PATH, error->message);
@@ -720,7 +739,7 @@ dbus_send_reload_signal(void)
 }
 
 static gboolean
-dbus_unregister_object(char *str)
+dbus_unregister_object(const char *str)
 {
 	gboolean ret = false;
 
@@ -738,13 +757,13 @@ dbus_unregister_object(char *str)
 }
 
 void
-dbus_remove_object(vrrp_t *vrrp)
+dbus_remove_object(const vrrp_t *vrrp)
 {
 	dbus_unregister_object(vrrp->iname);
 }
 
 static int
-handle_dbus_msg(__attribute__((unused)) thread_t *thread)
+handle_dbus_msg(__attribute__((unused)) thread_ref_t thread)
 {
 	dbus_queue_ent_t *ent;
 	char recv_buf;
@@ -806,11 +825,11 @@ handle_dbus_msg(__attribute__((unused)) thread_t *thread)
 				ent->reply = DBUS_SUCCESS;
 			}
 		}
-		if (write(dbus_out_pipe[1], ent, 1) != 1)
+		if (write(dbus_out_pipe[1], &recv_buf, 1) != 1)
 			log_message(LOG_INFO, "Write from main thread to DBus thread failed");
 	}
 
-	thread_add_read(master, handle_dbus_msg, NULL, dbus_in_pipe[0], TIMER_NEVER);
+	thread_add_read(master, handle_dbus_msg, NULL, dbus_in_pipe[0], TIMER_NEVER, false);
 
 	return 0;
 }
@@ -818,61 +837,67 @@ handle_dbus_msg(__attribute__((unused)) thread_t *thread)
 void
 dbus_reload(list o, list n)
 {
-	element e1, e2, e3;
-	vrrp_t *vrrp_n, *vrrp_o, *vrrp_n3;
+	element e1, e2;
+	vrrp_t *vrrp_n, *vrrp_o;
 
-	if (!LIST_ISEMPTY(n)) {
-		for (e1 = LIST_HEAD(n); e1; ELEMENT_NEXT(e1)) {
-			char *n_name;
-			bool match_found;
+	if (!dbus_running)
+		return;
 
-			vrrp_n = ELEMENT_DATA(e1);
+	LIST_FOREACH(n, vrrp_n, e1) {
+		char *n_name;
+		bool match_found;
 
-			if (LIST_ISEMPTY(o)) {
-				dbus_create_object(vrrp_n);
-				continue;
-			}
+		n_name = IF_BASE_IFP(vrrp_n->ifp)->ifname;
 
-			n_name = IF_BASE_IFP(vrrp_n->ifp)->ifname;
+		/* Try and find an instance with same vrid/family/interface that existed before and now */
+		match_found = false;
+		LIST_FOREACH(o, vrrp_o, e2) {
+			if (vrrp_n->vrid == vrrp_o->vrid &&
+			    vrrp_n->family == vrrp_o->family &&
+			    !strcmp(n_name, IF_BASE_IFP(vrrp_o->ifp)->ifname)) {
+				/* If the old instance exists in the new config,
+				 * then the dbus object will exist */
+				if (!strcmp(vrrp_n->iname, vrrp_o->iname)) {
+					match_found = true;
+					g_hash_table_replace(objects, no_const_char_p(vrrp_n->iname), g_hash_table_lookup(objects, vrrp_o->iname));
+					break;
+				} else {
+					gpointer instance;
+					if ((instance = g_hash_table_lookup(objects, vrrp_o->iname))) {
+						g_hash_table_remove(objects, vrrp_o->iname);
+						g_hash_table_insert(objects, no_const_char_p(vrrp_n->iname), instance);
+					}
+					match_found = true;
+					break;
+				}
 
-			/* Try an find an instance with same vrid/family/interface that existed before and now */
-			for (e2 = LIST_HEAD(o), match_found = false; e2 && !match_found; ELEMENT_NEXT(e2)) {
-				vrrp_o = ELEMENT_DATA(e2);
+#if 0
+				/* The following was in the original code, but I can't work out
+				 * its purpose. Leaving it here for now in case it is really needed. */
 
-				if (vrrp_n->vrid == vrrp_o->vrid &&
-				    vrrp_n->family == vrrp_o->family &&
-				    !strcmp(n_name, IF_BASE_IFP(vrrp_o->ifp)->ifname)) {
-					/* If the old instance exists in the new config,
-					 * then the dbus object will exist */
-					if (!strcmp(vrrp_n->iname, vrrp_o->iname)) {
+				/* Check if the old instance name we found still exists
+				 * (but has a different vrid/family/interface) */
+				LIST_FOREACH(n, vrrp_n3, e3) {
+					if (!strcmp(vrrp_o->iname, vrrp_n3->iname)) {
 						match_found = true;
 						break;
 					}
-
-					/* Check if the old instance name we found still exists
-					 * (but has a different vrid/family/interface) */
-					for (e3 = LIST_HEAD(n); e3; ELEMENT_NEXT(e3)) {
-						vrrp_n3 = ELEMENT_DATA(e3);
-						if (!strcmp(vrrp_o->iname, vrrp_n3->iname)) {
-							match_found = true;
-							break;
-						}
-					}
 				}
+#endif
 			}
-
-			if (match_found)
-				continue;
-
-			dbus_create_object(vrrp_n);
 		}
+
+		if (match_found)
+			continue;
+
+		dbus_create_object(vrrp_n);
 	}
 
 	/* Signal we have reloaded */
 	dbus_send_reload_signal();
 
 	/* We need to reinstate the read thread */
-	thread_add_read(master, handle_dbus_msg, NULL, dbus_in_pipe[0], TIMER_NEVER);
+	thread_add_read(master, handle_dbus_msg, NULL, dbus_in_pipe[0], TIMER_NEVER, false);
 }
 
 bool
@@ -880,6 +905,10 @@ dbus_start(void)
 {
 	pthread_t dbus_thread;
 	sigset_t sigset, cursigset;
+	int flags;
+
+	if (dbus_running)
+		return false;
 
 	if (open_pipe(dbus_in_pipe)) {
 		log_message(LOG_INFO, "Unable to create inbound dbus pipe - disabling DBus");
@@ -889,15 +918,23 @@ dbus_start(void)
 		log_message(LOG_INFO, "Unable to create outbound dbus pipe - disabling DBus");
 		close(dbus_in_pipe[0]);
 		close(dbus_in_pipe[1]);
+		dbus_in_pipe[0] = -1;
+		dbus_out_pipe[0] = -1;
 		return false;
 	}
 
 	/* We don't want the main thread to block when using the pipes,
 	 * but the other side of the pipes should block. */
-	fcntl(dbus_in_pipe[1], F_SETFL, fcntl(dbus_in_pipe[1], F_GETFL) & ~O_NONBLOCK);
-	fcntl(dbus_out_pipe[0], F_SETFL, fcntl(dbus_out_pipe[0], F_GETFL) & ~O_NONBLOCK);
+	flags = fcntl(dbus_in_pipe[1], F_GETFL);
+	if (flags == -1 ||
+	    fcntl(dbus_in_pipe[1], F_SETFL, flags & ~O_NONBLOCK) == -1)
+		log_message(LOG_INFO, "Unable to set dbus thread in_pipe blocking - (%d - %m)", errno);
+	flags = fcntl(dbus_out_pipe[0], F_GETFL);
+	if (flags == -1 ||
+	    fcntl(dbus_out_pipe[0], F_SETFL, flags & ~O_NONBLOCK) == -1)
+		log_message(LOG_INFO, "Unable to set dbus thread out_pipe blocking - (%d - %m)", errno);
 
-	thread_add_read(master, handle_dbus_msg, NULL, dbus_in_pipe[0], TIMER_NEVER);
+	thread_add_read(master, handle_dbus_msg, NULL, dbus_in_pipe[0], TIMER_NEVER, false);
 
 	/* Initialise the thread termination semaphore */
 	sem_init(&thread_end, 0, 0);
@@ -912,6 +949,8 @@ dbus_start(void)
 	/* Reenable our signals */
 	pthread_sigmask(SIG_SETMASK, &cursigset, NULL);
 
+	dbus_running = true;
+
 	return true;
 }
 
@@ -922,6 +961,12 @@ dbus_stop(void)
 	int ret;
 	gchar *path;
 
+	if (!dbus_running)
+		return;
+
+	g_hash_table_foreach_remove(objects, remove_object, NULL);
+	objects = NULL;
+
 	if (global_connection != NULL) {
 		path = dbus_object_create_path_vrrp();
 		dbus_emit_signal(global_connection, path, DBUS_VRRP_INTERFACE, "VrrpStopped", NULL);
@@ -935,7 +980,7 @@ dbus_stop(void)
 
 	clock_gettime(CLOCK_REALTIME, &thread_end_wait);
 	thread_end_wait.tv_sec += 1;
-	while ((ret = sem_timedwait(&thread_end, &thread_end_wait)) == -1 && errno == EINTR) ;
+	while ((ret = sem_timedwait(&thread_end, &thread_end_wait)) == -1 && check_EINTR(errno));
 
 	if (ret == -1 ) {
 		if (errno == ETIMEDOUT)
@@ -947,6 +992,17 @@ dbus_stop(void)
 		log_message(LOG_INFO, "Released DBus");
 		sem_destroy(&thread_end);
 	}
+
+	dbus_running = false;
+
+	close(dbus_in_pipe[0]);
+	close(dbus_in_pipe[1]);
+	dbus_in_pipe[0] = -1;
+	dbus_in_pipe[0] = -1;
+	close(dbus_out_pipe[0]);
+	close(dbus_out_pipe[1]);
+	dbus_out_pipe[0] = -1;
+	dbus_out_pipe[0] = -1;
 }
 
 #ifdef THREAD_DUMP
