@@ -295,10 +295,9 @@ add_nexthops(ip_route_t *route, struct nlmsghdr *nlh, struct rtmsg *rtm)
 }
 
 /* Add/Delete IP route to/from a specific interface */
-static int
+static bool
 netlink_route(ip_route_t *iproute, int cmd)
 {
-	int status = 1;
 	struct {
 		struct nlmsghdr n;
 		struct rtmsg r;
@@ -510,12 +509,12 @@ netlink_route(ip_route_t *iproute, int cmd)
 	if (netlink_talk(&nl_cmd, &req.n) < 0) {
 #if HAVE_DECL_RTA_EXPIRES
 		/* If an expiry was set on the route, it may have disappeared already */
-		if (cmd != IPADDRESS_DEL || !(iproute->mask & IPROUTE_BIT_EXPIRES))
+		if (cmd != IPROUTE_DEL || !(iproute->mask & IPROUTE_BIT_EXPIRES))
 #endif
-			status = -1;
+			return true;
 	}
 
-	return status;
+	return false;
 }
 
 /* Add/Delete a list of IP routes */
@@ -529,10 +528,9 @@ netlink_rtlist(list rt_list, int cmd)
 	if (LIST_ISEMPTY(rt_list))
 		return;
 
-	for (e = LIST_HEAD(rt_list); e; ELEMENT_NEXT(e)) {
-		iproute = ELEMENT_DATA(e);
+	LIST_FOREACH(rt_list, iproute, e) {
 		if ((cmd == IPROUTE_DEL) == iproute->set) {
-			if (netlink_route(iproute, cmd) > 0)
+			if (!netlink_route(iproute, cmd))
 				iproute->set = (cmd == IPROUTE_ADD);
 			else
 				iproute->set = false;
@@ -1797,36 +1795,32 @@ err:
 }
 
 /* Try to find a route in a list */
-static bool
+static ip_route_t *
 route_exist(list l, ip_route_t *iproute)
 {
 	ip_route_t *ipr;
 	element e;
 
-	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
-		ipr = ELEMENT_DATA(e);
-
-		/* The kernel's key to a route is (to, tos, preference, table, via) */
+	LIST_FOREACH(l, ipr, e) {
+		/* The kernel's key to a route is (to, tos, preference, table) */
 		if (IP_ISEQ(ipr->dst, iproute->dst) &&
-		    !ipr->via == !iproute->via &&
-		    (!ipr->via || IP_ISEQ(ipr->via, iproute->via)) &&
 		    ipr->dst->ifa.ifa_prefixlen == iproute->dst->ifa.ifa_prefixlen &&
 		    (!((ipr->mask ^ iproute->mask) & IPROUTE_BIT_METRIC)) &&
 		    (!(ipr->mask & IPROUTE_BIT_METRIC) ||
 		     ipr->metric == iproute->metric) &&
 		    ipr->table == iproute->table) {
 			ipr->set = iproute->set;
-			return true;
+			return ipr;
 		}
 	}
-	return false;
+	return NULL;
 }
 
 /* Clear diff routes */
 void
 clear_diff_routes(list l, list n)
 {
-	ip_route_t *iproute;
+	ip_route_t *iproute, *new_iproute;
 	element e;
 
 	/* No route in previous conf */
@@ -1842,7 +1836,7 @@ clear_diff_routes(list l, list n)
 
 	LIST_FOREACH(l, iproute, e) {
 		if (iproute->set) {
-			if (!route_exist(n, iproute)) {
+			if (!(new_iproute = route_exist(n, iproute))) {
 				log_message(LOG_INFO, "ip route %s/%d ... , no longer exist"
 						    , ipaddresstos(NULL, iproute->dst), iproute->dst->ifa.ifa_prefixlen);
 				netlink_route(iproute, IPROUTE_DEL);
@@ -1850,8 +1844,19 @@ clear_diff_routes(list l, list n)
 			else {
 				/* There are too many route options to compare to see if the
 				 * routes are the same or not, so just replace the existing route
-				 * with the new one. */
-				netlink_route(iproute, IPROUTE_REPLACE);
+				 * with the new one.
+				 * We try replacing the route, but if, for example, it has a src
+				 * address that is a new VIP, then the route won't be able to be
+				 * added (replaced) now. In this case delete the old route, mark
+				 * it as not set, and then it will be added later when any new
+				 * routes are added. */
+				netlink_error_ignore = EINVAL;
+				if (netlink_route(new_iproute, IPROUTE_REPLACE)) {
+					netlink_error_ignore = 0;
+					netlink_route(iproute, IPROUTE_DEL);
+					new_iproute->set = false;
+				} else
+					netlink_error_ignore = 0;
 			}
 		}
 	}
@@ -1869,7 +1874,7 @@ reinstate_static_route(ip_route_t *route)
 {
 	char buf[256];
 
-	route->set = (netlink_route(route, IPROUTE_ADD) > 0);
+	route->set = !netlink_route(route, IPROUTE_ADD);
 
 	format_iproute(route, buf, sizeof(buf));
 	log_message(LOG_INFO, "Restoring deleted static route %s", buf);
