@@ -34,7 +34,6 @@
 #include <net/if.h>
 #endif
 
-#include <xtables.h>
 #include <libiptc/libiptc.h>
 #include <libiptc/libip6tc.h>
 #ifdef _HAVE_LIBIPSET_
@@ -119,18 +118,6 @@ static void* libip4tc_handle;
 static void* libip6tc_handle;
 #endif
 
-#ifdef _LIBXTABLES_DYNAMIC_
-#include <dlfcn.h>
-
-/* The addresses of the functions we want */
-int (*xtables_insmod_addr)(const char *, const char *, bool);
-
-/* We can make it look as though normal linking is being used */
-#define xtables_insmod (*xtables_insmod_addr)
-
-static void *libxtables_handle;
-#endif
-
 static void
 set_iface(char *vianame, unsigned char *mask, const char *iface)
 {
@@ -183,7 +170,7 @@ int ip4tables_is_chain(struct iptc_handle* handle, const char* chain_name)
 	return iptc_is_chain(chain_name, handle);
 }
 
-int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name, unsigned int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint8_t type, int cmd, bool force)
+int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name, unsigned int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint8_t type, int cmd, uint8_t flags, bool force)
 {
 	size_t size;
 	struct ipt_entry *fw;
@@ -219,6 +206,8 @@ int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name,
 		memset ( &fw->ip.dmsk, 0xff, sizeof(fw->ip.dmsk));
 	}
 
+	fw->ip.invflags = flags;
+
 	if (in_iface)
 		set_iface(fw->ip.iniface, fw->ip.iniface_mask, in_iface);
 	if (out_iface)
@@ -315,7 +304,7 @@ int ip6tables_is_chain(struct ip6tc_handle* handle, const char* chain_name)
 	return ip6tc_is_chain(chain_name, handle);
 }
 
-int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name, unsigned int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint8_t type, int cmd, bool force)
+int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name, unsigned int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint8_t type, int cmd, uint8_t flags, bool force)
 {
 	size_t size;
 	struct ip6t_entry *fw;
@@ -346,9 +335,11 @@ int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name
 
 	if ( dst_ip_address && dst_ip_address->ifa.ifa_family != AF_UNSPEC ) {
 		memcpy(&fw->ipv6.dst, &dst_ip_address->u.sin6_addr, sizeof ( dst_ip_address->u.sin6_addr ) );
-		memset ( &fw->ipv6.dmsk, 0xff, sizeof(fw->ipv6.smsk));
+		memset ( &fw->ipv6.dmsk, 0xff, sizeof(fw->ipv6.dmsk));
 	}
 
+	fw->ipv6.invflags = flags;
+
 	if (in_iface)
 		set_iface(fw->ipv6.iniface, fw->ipv6.iniface_mask, in_iface);
 	if (out_iface)
@@ -409,74 +400,6 @@ int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name
 	return 0 ;
 }
 
-#ifdef _HAVE_LIBIPTC_
-#ifdef _LIBXTABLES_DYNAMIC_
-static
-bool xtables_load(void)
-{
-	if (libxtables_handle)
-		return true;
-
-	if (!(libxtables_handle = dlopen("libxtables.so", RTLD_NOW)) &&
-	    !(libxtables_handle = dlopen(XTABLES_LIB_NAME, RTLD_NOW))) {
-		log_message(LOG_INFO, "Unable to load xtables library - %s", dlerror());
-		return false;
-	}
-
-	if (!(xtables_insmod_addr = dlsym(libxtables_handle, "xtables_insmod"))) {
-		log_message(LOG_INFO, "Failed to dynamic link xtables_insmod - %s", dlerror());
-		dlclose(libxtables_handle);
-		libxtables_handle = NULL;
-
-		return false;
-	}
-
-	return true;
-}
-
-void xtables_unload(void)
-{
-	if (!libxtables_handle)
-		return;
-
-	dlclose(libxtables_handle);
-	libxtables_handle = NULL;
-}
-#endif
-
-bool
-load_xtables_module(const char *module,
-#ifndef _LIBXTABLES_DYNAMIC_
-					__attribute__((unused))
-#endif
-								const char *function)
-{
-	struct sigaction act, old_act;
-	bool res = true;
-
-#ifdef _LIBXTABLES_DYNAMIC_
-	if (!libxtables_handle && !xtables_load()) {
-		log_message(LOG_INFO, "Module %s cannot be loaded; not using %s", module, function);
-		return false;
-	}
-#endif
-
-	/* Enable SIGCHLD since xtables_insmod forks/execs modprobe */
-	act.sa_handler = SIG_DFL;
-	sigemptyset(&act.sa_mask);
-	act.sa_flags = 0;
-
-	sigaction(SIGCHLD, &act, &old_act);
-
-	if (xtables_insmod(module, NULL, true))
-		res = false;
-
-	sigaction(SIGCHLD, &old_act, NULL);
-
-	return res;
-}
-#endif
-
 #ifdef _HAVE_LIBIPSET_
 #ifndef IP_SET_OP_VERSION	/* Exposed to userspace from Linux 3.4 */
 				/* Copied from <linux/netfilter/ipset/ip_set.h> */
@@ -630,7 +553,7 @@ get_set_byname(const char *setname, struct xt_set_info *info, unsigned family, b
 #endif
 }
 
-int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsigned int rulenum, uint8_t dim, uint8_t src_dst, const char* target_name, const char* set_name, uint16_t protocol, uint8_t param, int cmd, bool ignore_errors)
+int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsigned int rulenum, uint8_t dim, uint8_t src_dst, const char* target_name, const ip_address_t *src_ip_address, const ip_address_t *dst_ip_address, const char* set_name, uint16_t protocol, uint8_t param, int cmd, bool ignore_errors)
 {
 	size_t size;
 	struct ipt_entry *fw;
@@ -662,6 +585,18 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 
 	fw->target_offset = XT_ALIGN(sizeof(struct ipt_entry));
 
+	if (src_ip_address && src_ip_address->ifa.ifa_family != AF_UNSPEC)
+	{
+		fw->ip.src.s_addr = src_ip_address->u.sin.sin_addr.s_addr;
+		fw->ip.smsk.s_addr = 0xffffffff;
+	}
+
+	if (dst_ip_address && dst_ip_address->ifa.ifa_family != AF_UNSPEC)
+	{
+		fw->ip.dst.s_addr = dst_ip_address->u.sin.sin_addr.s_addr;
+		fw->ip.dmsk.s_addr = 0xffffffff;
+	}
+
 	// set
 	match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
 	match->u.match_size = XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(*setinfo));
@@ -753,7 +688,7 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 	return 0;
 }
 
-int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, unsigned int rulenum, uint8_t dim, uint8_t src_dst, const char* target_name, const char* set_name, uint16_t protocol, uint8_t param, int cmd, bool ignore_errors)
+int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, unsigned int rulenum, uint8_t dim, uint8_t src_dst, const char* target_name, const ip_address_t *src_ip_address, const ip_address_t *dst_ip_address, const char* set_name, uint16_t protocol, uint8_t param, int cmd, bool ignore_errors)
 {
 	size_t size;
 	struct ip6t_entry *fw;
@@ -786,6 +721,18 @@ int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, uns
 
 	fw = (struct ip6t_entry*)MALLOC(size);
 
+	if (src_ip_address && src_ip_address->ifa.ifa_family != AF_UNSPEC) {
+//		memcpy(&fw->ipv6.src, &src_ip_address->u.sin6_addr, sizeof(src_ip_address->u.sin6_addr));
+		fw->ipv6.src = src_ip_address->u.sin6_addr;
+		memset(&fw->ipv6.smsk, 0xff, sizeof(fw->ipv6.smsk));
+	}
+
+	if ( dst_ip_address && dst_ip_address->ifa.ifa_family != AF_UNSPEC ) {
+//		memcpy(&fw->ipv6.dst, &dst_ip_address->u.sin6_addr, sizeof ( dst_ip_address->u.sin6_addr ) );
+		fw->ipv6.dst = dst_ip_address->u.sin6_addr;
+		memset(&fw->ipv6.dmsk, 0xff, sizeof(fw->ipv6.dmsk));
+	}
+
 	fw->target_offset = XT_ALIGN(sizeof(struct ip6t_entry));
 
 	// set
@@ -881,14 +828,17 @@ int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, uns
 #endif
 
 #ifdef _LIBIPTC_DYNAMIC_
-bool iptables_lib_init(void)
+bool
+iptables_lib_init(uint8_t family)
 {
-	if (!libip4tc_handle && block_ipv4) {
+	if (family == AF_INET) {
+		if (libip4tc_handle)
+			return true;
+
 		/* Attempt to open the ip4tc library */
 		if (!(libip4tc_handle = dlopen("libip4tc.so", RTLD_NOW)) &&
 		    !(libip4tc_handle = dlopen(IP4TC_LIB_NAME, RTLD_NOW))) {
 			log_message(LOG_INFO, "Unable to load ip4tc library - %s", dlerror());
-			using_libip4tc = false;
 		}
 		else if (!(iptc_init_addr = dlsym(libip4tc_handle, "iptc_init")) ||
 			 !(iptc_free_addr = dlsym(libip4tc_handle, "iptc_free")) ||
@@ -899,34 +849,34 @@ bool iptables_lib_init(void)
 			 !(iptc_commit_addr = dlsym(libip4tc_handle,"iptc_commit")) ||
 			 !(iptc_strerror_addr = dlsym(libip4tc_handle,"iptc_strerror"))) {
 			log_message(LOG_INFO, "Failed to dynamic link an iptc function - %s", dlerror());
-			using_libip4tc = false;
 			dlclose(libip4tc_handle);
 			libip4tc_handle = NULL;
 		}
+
+		return !!libip4tc_handle;
 	}
 
-	if (!libip6tc_handle && block_ipv6) {
-		/* Attempt to open the ip6tc library */
-		if (!(libip6tc_handle = dlopen("libip6tc.so", RTLD_NOW)) &&
-		    !(libip6tc_handle = dlopen(IP6TC_LIB_NAME, RTLD_NOW))) {
-			log_message(LOG_INFO, "Unable to load ip6tc library - %s", dlerror());
-			using_libip6tc = false;
-		}
-		else if (!(ip6tc_init_addr = dlsym(libip6tc_handle, "ip6tc_init")) ||
-			 !(ip6tc_free_addr = dlsym(libip6tc_handle, "ip6tc_free")) ||
-			 !(ip6tc_is_chain_addr = dlsym(libip6tc_handle,"ip6tc_is_chain")) ||
-			 !(ip6tc_insert_entry_addr = dlsym(libip6tc_handle,"ip6tc_insert_entry")) ||
-			 !(ip6tc_append_entry_addr = dlsym(libip6tc_handle,"ip6tc_append_entry")) ||
-			 !(ip6tc_delete_entry_addr = dlsym(libip6tc_handle,"ip6tc_delete_entry")) ||
-			 !(ip6tc_commit_addr = dlsym(libip6tc_handle,"ip6tc_commit")) ||
-			 !(ip6tc_strerror_addr = dlsym(libip6tc_handle,"ip6tc_strerror"))) {
-			log_message(LOG_INFO, "Failed to dynamic link an ip6tc function - %s", dlerror());
-			using_libip6tc = false;
-			dlclose(libip6tc_handle);
-			libip6tc_handle = NULL;
-		}
+	if (libip6tc_handle)
+		return true;
+
+	/* Attempt to open the ip6tc library */
+	if (!(libip6tc_handle = dlopen("libip6tc.so", RTLD_NOW)) &&
+	    !(libip6tc_handle = dlopen(IP6TC_LIB_NAME, RTLD_NOW))) {
+		log_message(LOG_INFO, "Unable to load ip6tc library - %s", dlerror());
+	}
+	else if (!(ip6tc_init_addr = dlsym(libip6tc_handle, "ip6tc_init")) ||
+		 !(ip6tc_free_addr = dlsym(libip6tc_handle, "ip6tc_free")) ||
+		 !(ip6tc_is_chain_addr = dlsym(libip6tc_handle,"ip6tc_is_chain")) ||
+		 !(ip6tc_insert_entry_addr = dlsym(libip6tc_handle,"ip6tc_insert_entry")) ||
+		 !(ip6tc_append_entry_addr = dlsym(libip6tc_handle,"ip6tc_append_entry")) ||
+		 !(ip6tc_delete_entry_addr = dlsym(libip6tc_handle,"ip6tc_delete_entry")) ||
+		 !(ip6tc_commit_addr = dlsym(libip6tc_handle,"ip6tc_commit")) ||
+		 !(ip6tc_strerror_addr = dlsym(libip6tc_handle,"ip6tc_strerror"))) {
+		log_message(LOG_INFO, "Failed to dynamic link an ip6tc function - %s", dlerror());
+		dlclose(libip6tc_handle);
+		libip6tc_handle = NULL;
 	}
 
-	return libip4tc_handle || libip6tc_handle;
+	return !!libip6tc_handle;
 }
 #endif
