@@ -29,6 +29,9 @@
 #include <fcntl.h>
 #include <string.h>
 #include <memory.h>
+#ifndef HAVE_SIGNALFD
+#include <signal.h>
+#endif
 
 #include "logger.h"
 #include "bitops.h"
@@ -39,7 +42,7 @@ static bool log_console = false;
 
 #ifdef ENABLE_LOG_TO_FILE
 /* File to write log messages to */
-char *log_file_name;
+const char *log_file_name;
 static FILE *log_file;
 bool always_flush_log_file;
 #endif
@@ -69,7 +72,7 @@ close_log_file(void)
 void
 open_log_file(const char *name, const char *prog, const char *namespace, const char *instance)
 {
-	char *file_name;
+	const char *file_name;
 
 	if (log_file) {
 		fclose(log_file);
@@ -84,11 +87,13 @@ open_log_file(const char *name, const char *prog, const char *namespace, const c
 	log_file = fopen_safe(file_name, "a");
 	if (log_file) {
 		int n = fileno(log_file);
-		fcntl(n, F_SETFD, FD_CLOEXEC | fcntl(n, F_GETFD));
-		fcntl(n, F_SETFL, O_NONBLOCK | fcntl(n, F_GETFL));
+		if (fcntl(n, F_SETFD, FD_CLOEXEC | fcntl(n, F_GETFD)) == -1)
+			log_message(LOG_INFO, "Failed to set CLOEXEC on log file %s", file_name);
+		if (fcntl(n, F_SETFL, O_NONBLOCK | fcntl(n, F_GETFL)) == -1)
+			log_message(LOG_INFO, "Failed to set NONBLOCK on log file %s", file_name);
 	}
 
-	FREE(file_name);
+	FREE_CONST(file_name);
 }
 
 void
@@ -97,21 +102,55 @@ flush_log_file(void)
 	if (log_file)
 		fflush(log_file);
 }
+
+void
+update_log_file_perms(mode_t umask_bits)
+{
+        if (log_file)
+                fchmod(fileno(log_file), (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH) & ~umask_bits);
+}
+#endif
+
+#ifndef HAVE_SIGNALFD
+static inline bool
+block_signals(sigset_t *cur_set)
+{
+	sigset_t block_set;
+
+	sigfillset(&block_set);
+	if (!sigprocmask(SIG_BLOCK, &block_set, cur_set))
+		return false;
+
+	/* Yes, we are logging without disabling signals,
+	 * but it would be useful to know that sigprocmask has
+	 * failed. The only error that could occur according
+	 * to sigprocmask(2) is EFAULT, which would be very
+	 * strange since the sigsets are on the stack. */
+	syslog(LOG_ERR, "%s", "sigprocmask failed in block_signals()");
+
+	return true;
+}
 #endif
 
 void
 vlog_message(const int facility, const char* format, va_list args)
 {
+#ifndef HAVE_SIGNALFD
+	sigset_t cur_set;
+	bool restore_signals = false;
+#endif
 #if !HAVE_VSYSLOG
 	char buf[MAX_LOG_MSG+1];
-
-	vsnprintf(buf, sizeof(buf), format, args);
 #endif
 
 	/* Don't write syslog if testing configuration */
 	if (__test_bit(CONFIG_TEST_BIT, &debug))
 		return;
 
+#if !HAVE_VSYSLOG
+	vsnprintf(buf, sizeof(buf), format, args);
+#endif
+
 	if (
 #ifdef ENABLE_LOG_TO_FILE
 	    log_file ||
@@ -133,10 +172,20 @@ vlog_message(const int facility, const char* format, va_list args)
 		char timestamp[64];
 		strftime(timestamp, sizeof(timestamp), "%c", &tm);
 
-		if (log_console && __test_bit(DONT_FORK_BIT, &debug))
+		if (log_console && __test_bit(DONT_FORK_BIT, &debug)) {
+#ifndef HAVE_SIGNALFD
+			if (!block_signals(&cur_set))
+				restore_signals = true;
+#endif
+
 			fprintf(stderr, "%s: %s\n", timestamp, buf);
+		}
 #ifdef ENABLE_LOG_TO_FILE
 		if (log_file) {
+#ifndef HAVE_SIGNALFD
+			if (!restore_signals && !block_signals(&cur_set))
+				restore_signals = true;
+#endif
 			fprintf(log_file, "%s: %s\n", timestamp, buf);
 			if (always_flush_log_file)
 				fflush(log_file);
@@ -144,11 +193,22 @@ vlog_message(const int facility, const char* format, va_list args)
 #endif
 	}
 
-	if (!__test_bit(NO_SYSLOG_BIT, &debug))
+	if (!__test_bit(NO_SYSLOG_BIT, &debug)) {
+#ifndef HAVE_SIGNALFD
+		if (!restore_signals && !block_signals(&cur_set))
+			restore_signals = true;
+#endif
+
 #if HAVE_VSYSLOG
 		vsyslog(facility, format, args);
 #else
 		syslog(facility, "%s", buf);
+#endif
+	}
+
+#ifndef HAVE_SIGNALFD
+	if (restore_signals)
+		sigprocmask(SIG_SETMASK, &cur_set, NULL);
 #endif
 }
 
