@@ -23,6 +23,9 @@
 #ifndef _PROCESS_H
 #define _PROCESS_H
 
+#ifdef _HAVE_SCHED_RT_
+#include <sched.h>
+#endif
 #include <sys/types.h>
 #include <sys/resource.h>
 
@@ -38,8 +41,13 @@ extern void set_process_priorities(
 #endif
 #endif
 				   int, int);
+#ifdef _HAVE_SCHED_RT_
+extern int set_process_cpu_affinity(cpu_set_t *, const char *);
+extern int get_process_cpu_affinity_string(cpu_set_t *, char *, size_t);
+#endif
 extern void reset_process_priorities(void);
-extern void set_child_rlimit(int, struct rlimit *);
+extern void set_child_rlimit(int, const struct rlimit *);
 extern pid_t local_fork(void);
 
+extern void set_max_file_limit(unsigned);
 #endif
