@@ -884,6 +884,7 @@ void
 thread_cleanup_master(thread_master_t * m)
 {
 	/* Unuse current thread lists */
+	m->current_event = NULL;
 	thread_destroy_rb(m, &m->read);
 	thread_destroy_rb(m, &m->write);
 	thread_destroy_rb(m, &m->timer);
@@ -1720,10 +1721,6 @@ thread_fetch_next_queue(thread_master_t *m)
 				}
 
 				if (__test_bit(LOG_DETAIL_BIT, &debug)) {
-					if (ep_ev->events & EPOLLHUP)
-						log_message(LOG_INFO, "Received EPOLLHUP for fd %d", ev->fd);
-					if (ep_ev->events & EPOLLERR)
-						log_message(LOG_INFO, "Received EPOLLERR for fd %d", ev->fd);
 					if (ep_ev->events & EPOLLRDHUP)
 						log_message(LOG_INFO, "Received EPOLLRDHUP for fd %d", ev->fd);
 				}
