@@ -18,23 +18,46 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _SIGNALS_H
 #define _SIGNALS_H
 
+#include "config.h"
+
+#include <signal.h>
+#include <stdbool.h>
+
+#include "scheduler.h"
+
+static inline int
+sigmask_func(int how, const sigset_t *set, sigset_t *oldset)
+{
+#ifdef _WITH_PTHREADS_
+    return pthread_sigmask(how, set, oldset);
+#else
+    return sigprocmask(how, set, oldset);
+#endif
+}
+
 /* Prototypes */
 extern int get_signum(const char *);
-extern void *signal_set(int signo, void (*func) (void *, int), void *);
-extern void *signal_ignore(int signo);
-extern void signal_handler_init(void);
-extern void signal_handler_child_clear(void);
+extern void signal_set(int, void (*) (void *, int), void *);
+extern void signal_ignore(int);
+extern int signal_handler_init(void);
 extern void signal_handler_destroy(void);
 extern void signal_handler_script(void);
-extern void signal_run_callback(void);
+extern void add_signal_read_thread(thread_master_t *);
+extern void cancel_signal_read_thread(void);
+#if HAVE_DECL_RLIMIT_RTTIME == 1
+extern void set_sigxcpu_handler(void);
+#endif
 
-extern int signal_rfd(void);
-extern void signal_pipe_close(int);
+extern void signal_fd_close(int);
+
+#ifdef THREAD_DUMP
+extern void register_signal_thread_addresses(void);
+#endif
 
 #endif
